/*----------------------------------------------------------------------------*/
/* File: Filehndl.c  File Manipulating functions                              */
/* Author: John Gann                                                          */
/* Start Date: 01 Oct 1990                                                    */
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <malloc.h>
#include <conio.h>
#include <direct.h>
#include <time.h>
#include <sys\timeb.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "criterr.h"
#include "filehndl.h"
#include "mnutils.h"


#define SEARCH_ENTIRE_DISK 0
#define SEARCH_CURRENT_DIR 1
#define REASONABLE_MAX_PATH 128

struct dir_struct
	{
	char dir_name[REASONABLE_MAX_PATH];
	struct dir_struct *nextDirNode;
	};

int get_files(char * path, char return_file_name[]);
int findfile(char * search_path, char * fileSpec, int mode);
int find_dirs(char * drive);
int does_file_exist(char * filename);
void remove_embedded_ws(char *str);
static void make_valid_dir(char * target, char  * source);

char * dir_name_ptrs[MAX_ALLOWABLE_DIRS];
char * file_name_ptrs[MAX_ALLOWABLE_FILES];

static struct dir_struct *topOfTree = (struct dir_struct *) NULL;

static char *drives[] = {
	"A:", "B:", "C:", "D:", "E:", "F:", "G:", "H:", "I:", "J:", "K:", "L:", "M:",
  "N:", "O:", "P:", "Q:", "R:", "S:", "T:", "U:", "V:", "W:", "X:", "Y:", "Z:"
   };

/*----------------------------------------------------------------------------*/
/* struct dir_struct *addNewDirectory (char * name)                           */
/*----------------------------------------------------------------------------*/
struct dir_struct *addNewDirectory (char * name)
{
struct dir_struct *tree;

/* first create a new branch */
tree = (struct dir_struct *) topOfTree;

/* first one ? */
if (tree == NULL)
	{
	tree = (struct dir_struct *) calloc (1, sizeof(struct dir_struct));
	if (tree == (struct dir_struct *) NULL) 
		{
		FreeDirList();
		NotifyUser(-1, -1, "addNewDirectory", "Out of Memory");
		topOfTree = (struct dir_struct *) tree;
		return (struct dir_struct *) NULL;
		}

	topOfTree = (struct dir_struct *) tree;
	}
else
	{
	while (tree->nextDirNode != NULL)
		{
		tree = tree->nextDirNode;
		}
	tree->nextDirNode =(struct dir_struct *) calloc(1, sizeof(struct dir_struct));
	if (tree->nextDirNode == (struct dir_struct *) NULL) 
		{
		FreeDirList();
		NotifyUser(-1, -1, "ERROR", "Out of Memory");
		return (struct dir_struct *) NULL;
		}

	tree = tree->nextDirNode;
	}

strcpy(tree->dir_name, name);
tree->nextDirNode = (struct dir_struct *) NULL;

return (struct dir_struct *) tree;
}



/*----------------------------------------------------------------------------*/
/* void dirsInPath(char *pathName)                     */
/*----------------------------------------------------------------------------*/
void dirsInPath(char *pathName)
{
struct find_t buffer;
char savePath[REASONABLE_MAX_PATH];
int len = strlen(pathName);
char name[REASONABLE_MAX_PATH];


if (pathName[len] != '\\')
  {
	strcat(pathName, "\\");
	}

addNewDirectory(pathName);

strcpy(savePath, pathName);
strcat(pathName, "*.*");

/* Now traverse subdirectories */
/* first print all files in this directory */
strcpy(pathName, savePath);
strcat(pathName, "*.*");
if (!(_dos_findfirst(pathName, _A_SUBDIR, &buffer)))
	{
	if (buffer.name[0] != '.' && buffer.attrib == _A_SUBDIR)
		{
		strcpy(name, savePath);
		strcat(name, buffer.name);
		dirsInPath(name);   /* recursion here */
		}

	while (!(_dos_findnext(&buffer)))
		{
		if (buffer.name[0] != '.' && buffer.attrib == _A_SUBDIR)
			{
			strcpy(name, savePath);
			strcat(name, buffer.name);
			dirsInPath(name);   /* recursion here */
			}
		}
	}
}






/*----------------------------------------------------------------------------*/
/* Free the linked list                                                       */
/*----------------------------------------------------------------------------*/
void FreeDirList(void)
{
struct dir_struct * top, * A, * B, * C;
A = top = topOfTree;

if (A== NULL)   /* Nothing to free */
	return ;

B = top->nextDirNode;

if (B == NULL)
	{
	free(A);         /* There are no links to free */
	topOfTree = (struct dir_struct *) NULL;
	return ;
	}
else
	{
	C = B->nextDirNode;
	if (C == NULL)
		{
		free(B);
		free(A);
		topOfTree = (struct dir_struct *) NULL;               /* 11 NOV 90 JTG */
		B = NULL;                       /* 11 NOV 90 JTG */
		return;
		}
	/* traverse linked list */
	while (C->nextDirNode != (struct dir_struct *) NULL)
		{
		B = A->nextDirNode;
		C = B->nextDirNode;
		if (C != NULL)
			{
			A->nextDirNode = C;
			}

		free(B);
		}

	free(A->nextDirNode);         /* Free last Node */
	free(top);         /* Free the top Node */
	topOfTree = (struct dir_struct *) NULL;
	}
}



/*----------------------------------------------------------------------------*/
/* Allow selection of a directory name from a list                            */
/*----------------------------------------------------------------------------*/
int get_dir(char * drive_name, char return_dir_name[])
{
struct dir_struct * top;
int dir_count = 0;

char drive[_MAX_DRIVE];
char dir[REASONABLE_MAX_PATH];
char fname[15];
char ext[15];

fm_desc * dir_lf;

int retv = 0;
int slots, cur_slot = 0, top_slot = 0;

int max_dir_length = 0;

if (!find_dirs(drive_name))
	{
	NotifyUser(-1, -1, "get_dirs():", "find_dirs returning FALSE");
	return FALSE;
	}

/* traverse linked list */
top = (struct dir_struct *) topOfTree;
while (top != NULL && dir_count < MAX_ALLOWABLE_DIRS)
	{
	dir_name_ptrs[dir_count++] = top->dir_name;
  max_dir_length = (int) max(max_dir_length, (int) strlen(top->dir_name));
	top = top->nextDirNode;
	if (dir_count >= MAX_ALLOWABLE_DIRS)
		NotifyUser(-1, -1, "Too Many Directories", "Only a Partial List of Directories");
	}

/* create the list */
dir_lf = CreateListForm( "Select a Directory", slots = 7, -1, max_dir_length+2, -1, 0);

/* Use ALT Key */
dir_lf->ob_lst [  9 ].state = 8;
dir_lf->ob_lst [ 10 ].state = 8;

/* open a form to put the list on */
OpenForm( dir_lf, -1, -1);

retv = ProcessListForm( dir_lf, dir_name_ptrs, dir_count,
	  	&top_slot, &cur_slot );

CloseForm(dir_lf);
DestroyForm(dir_lf);

if (retv != NOT_SET && retv != slots + 3)
	{
	_splitpath(dir_name_ptrs[cur_slot], drive, dir, fname, ext);
	sprintf(return_dir_name, "%s", dir);
	return TRUE;
	}

return FALSE;
}





/*------------------------------------------------------------------*/
/* Return a pointer to file spec, support help and cancel           */
/*------------------------------------------------------------------*/
int file_getter(char * path, char * prompt, char * default_path, int mode)
{
int retv;
ftag * tag;
char buffer[REASONABLE_MAX_PATH];
char drive[_MAX_DRIVE];
char dir[REASONABLE_MAX_PATH];
char fname[15];
char ext[15];
char cwd[REASONABLE_MAX_PATH];
char this_dir[REASONABLE_MAX_PATH];
char temp_path[REASONABLE_MAX_PATH];
char save_default_path[REASONABLE_MAX_PATH];
static int new_disk = -1;
int file_exists;
char drive_found = FALSE;
int i, disks;
fm_desc *path_fm;
char * frf = "path20.frf";

/*
*		Ob		Item
*		0			Disk Drive Search Button
*		1			Directory  Search Button
*		2			FileSpec   Search Button
*		3			Okay Button
*		4			Default Button
*		5			Cancel Button
*		6			Help Button
*		7			Find Button
*		8			Disk Drive Edit field 
*		9			Directory  Edit field 
*		10		FileSpec   Edit field 
*/

/* save the default path */
strcpy(save_default_path, default_path);

/* set up form */
_splitpath(strupr(path), drive, dir, fname, ext);
strcat(fname, ext);

if ( strlen ( drive ) == 0 ) 
   strcpy ( drive, drives [ _getdrive ()-1 ] );

if ( ( path_fm = loadFRF ( frf ) ) == (fm_desc *) NULL ) 
	{
	return ( FALSE );
	}

tag = (ftag *) path_fm->ob_lst[8].tag;
strcpy(tag->fld, drive);
tag = (ftag *) path_fm->ob_lst[9].tag;
strcpy(tag->fld, dir);
tag = (ftag *) path_fm->ob_lst[10].tag;
strcpy(tag->fld, fname);

path_fm->title = prompt;

OpenForm( path_fm, 1, 32);

do
	{
	retv = ProcessObList ( path_fm->ob_lst, path_fm->ob_cnt, 0 );

	switch ( retv )
		{
		case 0: /* Drive Search Button */
			/* save the drive */
			tag = (ftag *) path_fm->ob_lst[8].tag;
			strcpy(drive, tag->fld);
			
			if (new_disk == -1)
				{
				_dos_getdrive(&new_disk);
				new_disk--;
				}

			sprintf(buffer, "%c:", 'A' + new_disk);
			new_disk = get_disk(buffer);
			if (new_disk >= 0)
				{
				sprintf(buffer, "%c:", 'A' + new_disk);
				if (stricmp(buffer, drive))      /* Is this drive different ? */
					{
					tag = (ftag *) path_fm->ob_lst[9].tag;
					strcpy(tag->fld, "\\");
					DrawTaggedOb(&path_fm->ob_lst[9]);
					}
				tag = (ftag *) path_fm->ob_lst[8].tag;
 				strcpy(tag->fld, buffer);
				DrawTaggedOb(&path_fm->ob_lst[8]);
				}
			break;

		case 1: /* Directory Search Button */
			/* Get the current Drive */
			tag = (ftag *) path_fm->ob_lst[8].tag;
			strcpy(this_dir, tag->fld);

			if (get_dir(this_dir, buffer))
				{
				tag = (ftag *) path_fm->ob_lst[9].tag;
 				strcpy(tag->fld, buffer);
				DrawForm(path_fm);
				}
			break;

		case 2:  /* Filespec Search Button */
			FlushMsgQueue();
			tag = (ftag *) path_fm->ob_lst[8].tag;
 			strcpy(drive, tag->fld);
			tag = (ftag *) path_fm->ob_lst[9].tag;
			make_valid_dir(dir, tag->fld);
			_makepath(temp_path, drive, dir, "", "");
			tag = (ftag *) path_fm->ob_lst[10].tag;
			strcat(temp_path, tag->fld);

			/* if the user wants a PATH and not a single file, give it to em */
			if (mode != INPUT_PATH)
				{
				if (strchr(temp_path, '*'))
					{
					if (get_files(temp_path, buffer))
						{
						_splitpath(buffer, drive, dir, fname, ext);
						tag = (ftag *) path_fm->ob_lst[8].tag;
 						strcpy(tag->fld, drive);
						tag = (ftag *) path_fm->ob_lst[9].tag;
 						strcpy(tag->fld, dir);
						tag = (ftag *) path_fm->ob_lst[10].tag;
 						strcpy(tag->fld, fname);
 						strcat(tag->fld, ext);
						DrawForm(path_fm);
						}
					}
				}
			
			break;


		case 3 :     /* Okay */
			/* split the filename and suffix back out */
			tag = (ftag *) path_fm->ob_lst[10].tag;       // filename
 			strcpy(fname, tag->fld);
			strcpy(temp_path, fname);
			_splitpath(temp_path, drive, dir, fname, ext);

			tag = (ftag *) path_fm->ob_lst[8].tag;                // drive
 			strcpy(drive, tag->fld);

			tag = (ftag *) path_fm->ob_lst[9].tag;         // directory
			make_valid_dir(dir, tag->fld);

			_makepath(temp_path, drive, dir, fname, ext);

			/* -1 = CEH, TRUE = File exists, FALSE = !file_exists  */
			if ((file_exists = does_file_exist(temp_path)) == -1)
				{
				retv = 99;
				break;
				}

			switch (mode)
				{
				case NO_CHECKS:
					strcpy(path, temp_path);
					CloseForm(path_fm);
					DestroyForm(path_fm);
					FreeDirList();         // 10 Sep 91 JTG
					return TRUE;
					break;
					
				case INPUT:
					if (file_exists)
						{
						if (strchr(temp_path, '*'))
							{
							/* Search directory for filespec */
							tag = (ftag *) path_fm->ob_lst[8].tag;
 							strcpy(drive, tag->fld);
							tag = (ftag *) path_fm->ob_lst[9].tag;
							make_valid_dir(dir, tag->fld);
							_makepath(temp_path, drive, dir, "", "");
							tag = (ftag *) path_fm->ob_lst[10].tag;
							strcat(temp_path, tag->fld);

							if (get_files(temp_path, buffer))
								{
								_splitpath(buffer, drive, dir, fname, ext);
								tag = (ftag *) path_fm->ob_lst[8].tag;
 								strcpy(tag->fld, drive);
								tag = (ftag *) path_fm->ob_lst[9].tag;
 								strcpy(tag->fld, dir);
								tag = (ftag *) path_fm->ob_lst[10].tag;
 								strcpy(tag->fld, fname);
 								strcat(tag->fld, ext);
								DrawForm(path_fm);
								}

							/* free the linked list here because it now contains 
               	only the current directory's contents 11 NOV 90 JTG */
							FreeDirList();
							}
	
						/* Can't return a wild card */
						if (strchr(temp_path, '*'))
							{
							retv = 99;
							}
						else
							{
							strcpy(path, temp_path);
							CloseForm(path_fm);
     					DestroyForm(path_fm);
							FreeDirList();         // 10 Sep 91 JTG
							return TRUE;
							}
						}   /* end of if file exists */
					else
						{
						NotifyUser(-1, -1, "Warning, This file does not exist.", temp_path);
						retv = 99;        /* force user to stay in loop */	
						}

					break;

				case INPUT_PATH:
					if (!file_exists)
						{
						NotifyUser(-1, -1, "Warning, This file does not exist.", temp_path);
						retv = 99;        /* force user to stay in loop */	
						}
					else
						{
						strcpy(path, temp_path);
						CloseForm(path_fm);
   					DestroyForm(path_fm);
						FreeDirList();         // 10 Sep 91 JTG
						return TRUE;
						}
					break;

				case OUTPUT:
					if (file_exists)
						{
						if (QueryUser(-1, -1, temp_path, 
                           "Already exists, do you want to replace it?"))
							{
							CloseForm(path_fm);
	   					DestroyForm(path_fm);
							FreeDirList();         // 10 Sep 91 JTG
							return FALSE;
							}
						else
							{
							strcpy(path, temp_path);
							CloseForm(path_fm);
	   					DestroyForm(path_fm);
							FreeDirList();         // 10 Sep 91 JTG
							return TRUE;
							}
						}
					strcpy(path, temp_path);
					CloseForm(path_fm);
   				DestroyForm(path_fm);
					FreeDirList();         // 10 Sep 91 JTG
					return TRUE;
					break;
				}  /* end of switch mode */
			break;

		case 4:        /* Default */
			_splitpath(strupr(save_default_path), drive, dir, fname, ext);
			strcat(fname, ext);
			tag = (ftag *) path_fm->ob_lst[8].tag;
                        if ( strlen ( drive ) == 0 ) 
                           strcpy ( drive, drives [ _getdrive ()-1 ] );
			strcpy(tag->fld, drive);
			tag = (ftag *) path_fm->ob_lst[9].tag;
			strcpy(tag->fld, dir);
			tag = (ftag *) path_fm->ob_lst[10].tag;
			strcpy(tag->fld, fname);
			DrawForm(path_fm);
			break;
			
		case 5 :     /* Cancel */
			CloseForm(path_fm);
			DestroyForm(path_fm);
			FreeDirList();         // 10 Sep 91 JTG
			return FALSE;
			break;
	
		case 6: 
			NotifyUser(-1, -1, "Yo", "You are Beyond Help");
			break;
	
		case 7:      /* Find */
			/* Find all occurences of filespec and set form dir and filespec */

			/* Get the current Drive */
			tag = (ftag *) path_fm->ob_lst[8].tag;
			strcpy(drive, tag->fld);

			/* Get the filespec */
			tag = (ftag *) path_fm->ob_lst[10].tag;
			strcpy(temp_path, tag->fld);

			/* Search drive for filespec */
			if (findfile(drive, temp_path, SEARCH_ENTIRE_DISK))
				{
				_splitpath(temp_path, drive, dir, fname, ext);

				tag = (ftag *) path_fm->ob_lst[9].tag;
 				strcpy(tag->fld, dir);

				/* if requesting a path don't wipe the wild card */
				if (mode != INPUT_PATH)
					{
					tag = (ftag *) path_fm->ob_lst[10].tag;
 					strcpy(tag->fld, fname);
 					strcat(tag->fld, ext);
					}
				DrawForm(path_fm);
				}
			break;

		case 8: /* Drive Edit Field */

			tag = (ftag *) path_fm->ob_lst[8].tag;
			strcpy(buffer, strupr(tag->fld));
			remove_embedded_ws(buffer);

			if ( strlen ( buffer ) == 0 ) strcpy ( buffer, "C:" );
			else
			if (buffer[strlen(buffer) - 1] != ':') strcat(buffer, ":");

			/* Check to see if the entered drive is actually available */
			drive_found = FALSE;
			disks = get_number_of_drives(drives);
			for (i=0; i<disks; i++)
				{
				if (!stricmp(buffer, drives[i]))
					{
					drive_found = TRUE;
					}
				}

			if (!drive_found)
				{
				NotifyUser(-1, -1, buffer, "No such Drive");
				tag = (ftag *) path_fm->ob_lst[8].tag;
				strcpy(tag->fld, drive);
				DrawTaggedOb(&path_fm->ob_lst[8]);
				break;
				}

			/* Is this a new drive ? */
			if (stricmp(buffer, drive))      /* Is this drive different ? */
				{
				tag = (ftag *) path_fm->ob_lst[9].tag;
				strcpy(tag->fld, "\\");
				DrawTaggedOb(&path_fm->ob_lst[9]);
				}

			/* Restore the contents of the drive field */
			tag = (ftag *) path_fm->ob_lst[8].tag;
			strcpy(tag->fld, buffer);
			DrawTaggedOb(&path_fm->ob_lst[8]);

			break;

		case 9: /* Directory Edit Field */

			/* Save where we are */
			if (getcwd(cwd, REASONABLE_MAX_PATH) == NULL)
				{
				NotifyUser(-1, -1, "Error", "Unable to get CWD");
				}

			/* See if we can CD to this new dir */
			strcpy(buffer, drive);
			tag = (ftag *) path_fm->ob_lst[9].tag;
			strcat(buffer, strupr(tag->fld));
			remove_embedded_ws(buffer);
			if ((strlen(tag->fld) > 1) && (buffer[strlen(buffer) - 1] == '\\'))
				buffer[strlen(buffer) - 1] = '\0';

			if (chdir(buffer))
				{
				NotifyUser(-1, -1, buffer, "Does Not Exist!");
				tag = (ftag *) path_fm->ob_lst[9].tag;
				strcpy(tag->fld, dir);
				DrawTaggedOb(&path_fm->ob_lst[9]);
				break;
				}
			else  /* Dir is OK so change back to where we were */
				{
				chdir(cwd);
				}

			tag = (ftag *) path_fm->ob_lst[9].tag;
			strcpy(buffer, strupr(tag->fld));
			if (buffer[strlen(buffer) - 1] != '\\')
				{
				strcat(buffer, "\\");
				strcpy(tag->fld, buffer);
				strcpy(tag->fld, strupr(buffer));
				DrawTaggedOb(&path_fm->ob_lst[9]);
				}
			break;

		case 10: /* FileSpec Edit Field */
			break;
	
		case NOT_SET:
				CloseForm(path_fm);
				DestroyForm(path_fm);
				FreeDirList();         // 10 Sep 91 JTG
				return FALSE;
			break;

		case USER_IGNORE:
			break;
		}					/* end of switch */

	/* Update the variables for the next loop */
	tag = (ftag *) path_fm->ob_lst[8].tag;
	strcpy(this_dir, tag->fld);
	tag = (ftag *) path_fm->ob_lst[9].tag;
	strcat(this_dir, tag->fld);
	tag = (ftag *) path_fm->ob_lst[10].tag;
	strcat(this_dir, tag->fld);
	_splitpath(this_dir, drive, dir, fname, ext);
	} while (retv != 3 && retv != 5);
}


/*------------------------------------------------------------------*/
/* Return index to disk drive selected A=0, B=1, C=2 etc            */
/*------------------------------------------------------------------*/
int get_disk(char * old_disk)
{
int retv;
int i, disks, cur_item;
fm_desc * drive_fm;
ob_desc * ob;
int w, h, x, y;
unsigned current_drive;

_dos_getdrive(&current_drive);

/* get the number and names of the logical disk drives on the system */
disks = get_number_of_drives(drives);

/* Create form with disk number of objects */
drive_fm = CreateForm(disks);

/* figure out sizes and dimensions */
w = (disks * charW * 9) + (2 * charW);
h = charH * 8;

/* initialize form */
SetRect( &drive_fm->area, 0, 0, w, h );
drive_fm->flags = IS_INIT;
drive_fm->attr  = attrForm1;
drive_fm->title = "Select Disk Drive";
ob = drive_fm->ob_lst;

/* Set up all the buttons */
for (i=0; i< disks; i++)
	{
	ob->attr  = attrCBut;
	ob->otyp  = obButton;
	ob->flags = IS_SELECTABLE|IS_EXITABLE|IS_INVERTIBLE|IS_EXCLUSIVE|IS_CHECKABLE;

  x = drive_fm->area.Xmin + (i * charW * 9) + charW;
  y = drive_fm->area.Ymax - charH*3 - 1;

  SetRect( &ob->area, x, y, x+charW*3, y-charH*2);
  ob->key = drives[i][0];
  ob->state = 8;
  ob->label = (char *) drives[i];
  BxArea2Work( &ob->area, &ob->area, ob->attr );
	ob++;
	}

/* make sure the current drive is checked  */
for (i=0; i< disks; i++)
	{
	if (!stricmp(old_disk, drives[i]))
		{
		cur_item = i;
		drive_fm->ob_lst [ i ].flags |= IS_CHECKED;
		}
	}

OpenForm( drive_fm, 1, 32);

retv = ProcessObList ( drive_fm->ob_lst, drive_fm->ob_cnt, 0 );
	
CloseForm(drive_fm);
DestroyForm(drive_fm);
return (retv == -1) ? current_drive - 1: (int) (drives[retv][0]) - 'A';
}


/*----------------------------------------------------------------------------*/
/* Pick a file from a linked list                                             */
/*----------------------------------------------------------------------------*/
int get_files(char * path, char return_file_name[])
{
char drive[_MAX_DRIVE];
char dir[REASONABLE_MAX_PATH];
char fname[15];
char ext[15];

fm_desc *file_lf;
file_list fList;

int retv = 0;
int slots, cur_slot = 0, top_slot = 0;

int max_file_length = 0;

/* split the path for use later */
_splitpath(path, drive, dir, fname, ext);

Notify( -1, -1, "Scanning directory..." );
retv = CreateFileList( &fList, path, 1 );
NotifyOff();

/* check for possible error returns */
switch( retv )
	{
	case  1 : /* no files found */
		NotifyUser(-1, -1,  path, "No files found that match");
		return FALSE ;
		break;

	case -1 : /* out of memory */
		NotifyUser(-1, -1, "Error:", "Out of memory\nallocating file list" );
		return FALSE;
		break;
	}


if (checkCEH())
	{
	topOfTree = (struct dir_struct *) NULL;
	return FALSE;
	}

/* create the list */
file_lf = CreateListForm( path, slots = 7, -1, 14, -1, 0);

/* Use ALT Key */
file_lf->ob_lst [  9 ].state = 8;
file_lf->ob_lst [ 10 ].state = 8;

/* open a form to put the list on */
OpenForm( file_lf, -1, -1);

retv = ProcessListForm( file_lf, fList.fnames, fList.nfiles,
	  	&top_slot, &cur_slot );

CloseForm(file_lf);
DestroyForm(file_lf);

if (retv != NOT_SET && retv != slots + 3)
	{
	strcpy(return_file_name, drive);
	strcat(return_file_name, dir);
	strcat(return_file_name, file_lf->ob_lst[0].label);
	return TRUE;
	}

return FALSE;
}


/*----------------------------------------------------------------------------*/
/* Create a linked list of dir names                                          */
/*----------------------------------------------------------------------------*/
int find_dirs(char * drive)
{
static char last_drive[_MAX_DRIVE] = {(char) NULL};
char initialPath[REASONABLE_MAX_PATH];

/* See if we have already looked at this disk */
if (topOfTree != (struct dir_struct *) NULL)
	{
	/* Was this disk the same as last time */
	if (stricmp(last_drive, drive))
		{
		FreeDirList();
		ResetCritErr();
		Notify(-1, -1, "..scanning directories");
		strcpy(initialPath, drive);
		dirsInPath(initialPath);
		NotifyOff();
		}
	}
else
	{
	ResetCritErr();
	Notify(-1, -1, "..scanning directories");
	strcpy(initialPath, drive);
	dirsInPath(initialPath);
	NotifyOff();
	}


if (checkCEH())
	{
	topOfTree = (struct dir_struct *) NULL;
	return FALSE;
	}

/* save this drive name for later comparison */
strcpy(last_drive, drive);

return TRUE;
}





/*----------------------------------------------------------------------------*/
/* Create a linked list of file names                                         */
/*----------------------------------------------------------------------------*/
int findfile(char * search_path, char * fileSpec, int mode)
{
char buf[512];
struct dir_struct * top;
int file_count = 0;
struct find_t f_buf;
int retv = 0;
int slots, cur_slot = 0, top_slot = 0;
int max_length = 0;
fm_desc * find_lf;
char initialPath[REASONABLE_MAX_PATH];

/* find all the dirs on this search_path */
if (mode == SEARCH_ENTIRE_DISK) 
	{
	if (!find_dirs(search_path))
		{
		NotifyUser(-1, -1, "findfile():", "find_dirs returning FALSE");
		}
	}
else
	{
	top = (struct dir_struct *) topOfTree;
	if (top != NULL)
		{
		FreeDirList();
		}
	addNewDirectory(search_path);
	}

/* traverse linked list */
Notify( -1, -1, "Building List..." );
top = (struct dir_struct *) topOfTree;
while ((top != NULL) && 
       (file_count < MAX_ALLOWABLE_FILES) )
	{
	/* find the occurence of fileSpec in each directory */
	strcpy(initialPath, top->dir_name);
	strcat(initialPath, fileSpec);
	if (!(_dos_findfirst(initialPath, _A_NORMAL, &f_buf)))
		{
		file_name_ptrs[file_count] = (char *) malloc(strlen(top->dir_name) + 13);
		if (file_name_ptrs[file_count] == (char *) NULL)
			{
			while (file_count) 
				{
				free(file_name_ptrs[--file_count]);
				}
			NotifyUser(-1, -1, "findfiles():", "Out of Memory, allocating file list");
			NotifyOff();
			return FALSE;
			}

		strcpy(file_name_ptrs[file_count], top->dir_name);
		strcat(file_name_ptrs[file_count], f_buf.name);
		max_length = max(max_length, (int) strlen(file_name_ptrs[file_count]));
		file_count++;
	
		while (!(_dos_findnext(&f_buf)) && 
            (file_count < MAX_ALLOWABLE_FILES) )
			{
			file_name_ptrs[file_count] = (char *) malloc(strlen(top->dir_name) + 13);
			if (file_name_ptrs[file_count] == (char *) NULL)
				{
				while(file_count)
					{
					free(file_name_ptrs[--file_count]);
					}
				NotifyUser(-1, -1, "findfiles():", "Out of Memory, allocating file list");
				NotifyOff();
				return FALSE;
				}
	
			strcpy(file_name_ptrs[file_count], top->dir_name);
			strcat(file_name_ptrs[file_count], f_buf.name);
	  	max_length = max(max_length, (int) strlen(file_name_ptrs[file_count]));
			file_count++;
			if (file_count >= MAX_ALLOWABLE_FILES)
				{
				NotifyUser(-1, -1, "Too Many Files", "Only a Partial List of Files\nNarrow the Search Pattern");
				break;
				}
			}   /* while !_dos_findnext()  */

		}   /* end of if !_dos_findfirst ()  */
		
	top = top->nextDirNode;
	}      /* end of while (top != NULL)  */

NotifyOff();

if (!file_count)
	{
	sprintf(buf, "No Occurrences of %s \nfound in %s", fileSpec, search_path);
	NotifyUser(-1, -1, "FINDFILE", buf);
	return FALSE;
	}

/* create the list */
find_lf = CreateListForm( "Files Found", slots = 7, -1, max_length+4, -1, 0);
if (find_lf == NULL)
	{
	while(file_count)
		{
		free(file_name_ptrs[--file_count]);
		}
	NotifyUser(-1, -1, "FINDFILE", "Out of Memory Creating List Form");
	return FALSE;
	}

/* Use ALT Key */
find_lf->ob_lst [  9 ].state = 8;
find_lf->ob_lst [ 10 ].state = 8;

/* open a form to put the list on */
if (!OpenForm( find_lf, 1, -1))
	{
	while(file_count)
		{
		free(file_name_ptrs[--file_count]);
		}
	NotifyUser(-1, -1, "FINDFILE", "Out of Memory Opening Form");
	return FALSE;
	}

retv = ProcessListForm( find_lf, file_name_ptrs, file_count,
	  	&top_slot, &cur_slot );

CloseForm(find_lf);
DestroyForm(find_lf);
	
if (retv != NOT_SET && retv != slots + 3)
	{
	strcpy(fileSpec, file_name_ptrs[cur_slot]);
	while(file_count)
		{
		free(file_name_ptrs[--file_count]);
		}
	return TRUE;
	}

/* free up malloc'd memory */
while(file_count)
	{
	free(file_name_ptrs[--file_count]);
	}

return FALSE;
}

/*----------------------------------------------------------------------------*/
/* Check for existance of a file, using _dos calls and Crit Err Handler       */
/*----------------------------------------------------------------------------*/
int does_file_exist(char * path)
{
int file_exists;
struct find_t f_buf;

file_exists =  !(_dos_findfirst(path, _A_NORMAL, &f_buf));

if (checkCEH())
	{
	return -1;
	}

/* returns 0 if file found else DOS error code if not found */
return (file_exists) ? TRUE : FALSE;
}


/*----------------------------------------------------------------------------*/
// Make a valid directory name complete with trailing slash
/*----------------------------------------------------------------------------*/
static void make_valid_dir(char * target, char  * source)
{
int i = 0;
char * t = (char *) target;
char * s = (char *) source;

while ((*t++ = *s++) && i++ < REASONABLE_MAX_PATH);

if (target[strlen(target)-1] != '\\')  /* Add trailing backslash */
	{
	strcat(target, "\\");   /* if it is not present          */
	target[strlen(target)-1] = '\\';
	target[strlen(target)] = '\0';
	}
}


/*----------------------------------------------------------------------------*/
/* Return number of disk drives and fill in pointer list                      */
/*----------------------------------------------------------------------------*/
int get_number_of_drives(char * disk_names[])
{
int i;
int num_of_drives;
unsigned current_drive;
unsigned temp_drive;
int OK_drives = 0;

_dos_getdrive(&current_drive);
_dos_setdrive(current_drive, &num_of_drives);

for (i=1; i<=num_of_drives; i++)
	{
	_dos_setdrive((unsigned) i, &temp_drive);
	_dos_getdrive(&temp_drive);
	if (temp_drive == (unsigned) i)
		{
		sprintf(disk_names[OK_drives++],"%c:", 'A' + (unsigned) i - 1);
		}	
	}

_dos_setdrive((unsigned) current_drive, &temp_drive);

return OK_drives;
}



#define MAX_STRING 512
/*----------------------------------------------------------------------------*/
/* Strips multiple embedded and leading and trailing whitespace								*/
/*----------------------------------------------------------------------------*/
void remove_embedded_ws(char *str)
{
char *s;
char ch;
int i = 0;
char temp[MAX_STRING];
char copy[MAX_STRING];
int last_ch_was_a_space = FALSE;

if (strlen(str) > MAX_STRING)
	return;

strcpy(copy, str);
s = (char *) copy;
if (s == (char *) NULL)
	return;

strcpy(s, str);

while (ch = *s++)
	{
	if (isspace(ch))
		{
		/*convert all ws (tabs & newlines) to spaces*/
		ch = ' ';

		/* allow for one embedded space but no leading spaces */
		if (!last_ch_was_a_space && i)
			{
			temp[i++] = ch;
			}
		last_ch_was_a_space = TRUE;
		}
	else
		{
		temp[i++] = ch;
		last_ch_was_a_space = FALSE;
		}
	}
temp[i] = '\0';

/* now remove trailing ws */
if (temp[i-1] == ' ')
	temp[i-1] = '\0';

strcpy (str, temp);
}



/*----------------------------------------------------------------------------*/
// Check the critical error handler and report the error and return it
/*----------------------------------------------------------------------------*/
int checkCEH(void)
{
char buf[128];

if (is_critical_err)
	{
	switch(ce_err_code)
		{
		case 2:
		sprintf(buf, "Disk Drive %c: Not Ready", ce_drive_num + 'A');
		break;

		default:
		sprintf(buf, "Unknown Critical Error \nDOS Error Code %d", ce_err_code);
		break;
		}
	NotifyUser(-1, -1, "Yo", buf);
	ResetCritErr();
	}

return is_critical_err;
}
