/*----------------------------------------------------------------------------*/
// File: navcfg.c   USGS Navigation - nav setup program
// Author: John Gann
// Start Date: 3 Aug 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <time.h>
#include <math.h>
#include <fcntl.h>
#include <io.h>
#include <bios.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "nav_util.h"
#include "navcfg.h"
#include "criterr.h"
#include "dispasci.h"
#include "filehndl.h"
#include "mnutils.h"

// global data
char _SmartPortInstalled = FALSE;
fm_desc * mainMenu;
char DisplayingSomething = FALSE;
char RefreshRequired = FALSE;
int (*usrKeyTask)(ob_desc * ob);                       // used by DisplayAscii

static char * NULL_COM_MSG = "Null COM pointer";
static char * NULL_NAV_MSG = "Null NAV pointer";
static char * NULL_SETUP_MSG  = "Null Configuration";


/*----------------------------------------------------------------------------*/
/* void init(int argc, char *argv[])                                          */
/*----------------------------------------------------------------------------*/
int init_nav(int argc, char *argv[])
{
t_setup * config = QuerySetup();

// initialize the printer for streamed IO
setmode ( fileno ( stdprn ), O_TEXT );

/* Set time zone variable to GMT0 */
if (putenv("TZ=GMT0"))
	{
	printf("\nCan't Set environment variable TZ");
	}
else
	{
	tzset();
	}

/* Initialize Menuet stuff */
InitApp( argc, argv );

if (QueryColors() <= 1)
  {
  SetAttrTblColors ( 0, 15 );
  }
else
  {
  SetAttrTblColors ( config->fg, config->bg );
  }

/* Try to Load the MRF FILE */
if ( ( mainMenu = LoadMRF ( "navcfg.mrf" ) ) == NULL ) 
	{
	Puke("Error Loading MRF\n");
	}

OpenApp( NAV_VERSION , mainMenu, NULL);
OpenMsgQueue( 1 );

/* Set the Esc Key up to cause immediate exit */
SetEscape(ESC_ESCAPE|ESC_MOUSE);

/* Set the number of timer ticks between to something really fast  */
SetDblClickInt (2);

/* Turn off Ghosting */
SetWnGhosting(0);

// Install Crical Error Handler
InstallCEH();

return TRUE;
}







/*------------------------------------------------------------------*/
/* Give Not Yet Implemented message                                 */
/*------------------------------------------------------------------*/
int not_implemented(ob_desc * ob)
{
NotifyUser(-1, -1, ob->label, "Yo, this feature not yet implemented");
return TRUE;
}




/*------------------------------------------------------------------*/
// Return a pointer to the main menu
/*------------------------------------------------------------------*/
fm_desc *  QueryMenu(void)
{
return (fm_desc *) mainMenu;
}


/*------------------------------------------------------------------*/
// the Exit function
/*------------------------------------------------------------------*/
int doExit(ob_desc * ob)
{
Puke("Terminated by Operator");
return TRUE;
}



/*------------------------------------------------------------------*/
/* Assign sub menus and tasks to main menu items										*/
/*------------------------------------------------------------------*/
void InitNavMenus(void)
{
fm_desc * thisFm;

/* File SubMenu */
mainMenu->flags |= IS_MDTGEN;
mainMenu->ob_lst[0].flags |= IS_MENUED;
mainMenu->ob_lst[1].flags |= IS_MENUED;

thisFm = (fm_desc *) mainMenu->ob_lst[0].tag;            

thisFm->ob_lst[0].flags |= IS_EXECUTABLE;
thisFm->ob_lst[0].task = doSaveCfg;                // Save Configuration

thisFm->ob_lst[1].flags |= IS_EXECUTABLE;
thisFm->ob_lst[1].task = doListCfg;                // Dump configuration

thisFm->ob_lst[2].flags |= IS_EXECUTABLE;
thisFm->ob_lst[2].task = doExit;                   // Exit form

// Edit
thisFm = (fm_desc *) mainMenu->ob_lst[1].tag;            

thisFm->ob_lst[0].flags |= IS_EXECUTABLE;
thisFm->ob_lst[0].task = doVideoCfg;                  // Video

thisFm->ob_lst[1].flags |= IS_EXECUTABLE;
thisFm->ob_lst[1].task = doRelayCfg;                  // Relays

thisFm->ob_lst[2].flags |= IS_EXECUTABLE;
thisFm->ob_lst[2].task = doColorFm;           // foreground color

thisFm->ob_lst[3].flags |= IS_EXECUTABLE;
thisFm->ob_lst[3].task = doColorFm;           // Background color

mainMenu->ob_lst[2].flags |= IS_EXECUTABLE;
mainMenu->ob_lst[2].task   = doExit;
return;
}



/*----------------------------------------------------------------------------*/
// Terminate from anywhere
/*----------------------------------------------------------------------------*/
void Puke(char * msg)
{
RemoveCEH();                      // remove Dos critical error handler
write_setup();                    // update configuration

/**********************************************************************
write_video(VIDEO_FILE_NAME);     // update the video file settings
write_nav(NAV_FILE_NAME);         // update last position speed, etc
write_line_file();                // update the line list
write_offsets(OFFSET_FILE_NAME);  // update the vessel offset file
write_station_file();             // update the station list
write_com_file(COM_FILE_NAME);    // update the serial port assignments
write_graf(GRAF_FILE_NAME);       // update the graphics configuration
*************************************************************************/

CloseMsgQueue();                  // Shut down menuet and Metagraphics
TermApp(msg);                     // Close the application
}



/*----------------------------------------------------------------------------*/
// Get the Relay Board Config
/*----------------------------------------------------------------------------*/
int doRelayCfg ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "relaycfg.frf";
t_setup * config = QuerySetup();
ftag * addressTag;

if (config == (t_setup *) NULL)
	{
	NotifyUser(-1, -1, NULL_SETUP_MSG, frf);
  return FALSE;
	}

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

// set up form
if (config->relay.enabled)
	{
	fm->ob_lst [ 2 ].flags |=  IS_CHECKED;
	}
else
	{
	fm->ob_lst [ 2 ].flags &= ~IS_CHECKED;
	}

addressTag   = (ftag *) fm->ob_lst[3].tag;
sprintf(addressTag->fld, "%d",  config->relay.address);

OpenForm ( fm, ob->area.Xmin, ob->area.Ymin ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the fields
			config->relay.enabled = (fm->ob_lst [ 2 ].flags & IS_CHECKED) ? 1 : 0;
			config->relay.address = (unsigned int) atoi(addressTag->fld);
			break;

		case CANCEL:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL - 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}




/*----------------------------------------------------------------------------*/
// Get the Vidoe Board Enabled Flag
/*----------------------------------------------------------------------------*/
int doVideoCfg ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "videocfg.frf";
t_setup * config = QuerySetup();

if (config == (t_setup *) NULL)
	{
	NotifyUser(-1, -1, NULL_SETUP_MSG, frf);
  return FALSE;
	}

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

// set up form
if (config->videoEnabled)
	{
	fm->ob_lst [ 2 ].flags |=  IS_CHECKED;
	}
else
	{
	fm->ob_lst [ 2 ].flags &= ~IS_CHECKED;
	}

OpenForm ( fm, ob->area.Xmin, ob->area.Ymin ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the fields
			config->videoEnabled = (fm->ob_lst [ 2 ].flags & IS_CHECKED) ? 1 : 0;
			break;

		case CANCEL:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL - 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}


/*----------------------------------------------------------------------------*/
// Save the configuration
/*----------------------------------------------------------------------------*/
int doSaveCfg ( ob_desc * ob )
{
int retv;

if (retv = write_setup())
	{
	NotifyUser(-1, -1, "Configuration Saved", CFG_FILE_NAME);
	}
else
	{
	NotifyUser(-1, -1, "Error Saving Configuration", CFG_FILE_NAME);
	}
return retv;
}


/*----------------------------------------------------------------------------*/
/* Save the com file from a global array of com structures                    */
/*----------------------------------------------------------------------------*/
int doListCfg( ob_desc * ob)
{
FILE *fp;
int  i;
t_setup * config;
char tmpname[_MAX_PATH] = {"nvXXXXXX"};

// Create a temp file name to store the output to.
if (mktemp(tmpname) == NULL)
	{
	return UNABLE_TO_CREATE_TMPFILE;
	}
else
	{
	if ((fp = fopen(tmpname, "w")) == (FILE *) NULL)
		{
		return BAD_FILE_OPEN;
		}
	}


fprintf(fp, "Initialized       %d\n", config->Initialized);
fprintf(fp, "Coordinate System %d\n", config->CoordSys);
fprintf(fp, "Angle Type %d\n", config->AngleType);
fprintf(fp, "Horizontal Units %d\n", config->HorizUnits);
fprintf(fp, "Decimal Accuracy %d\n", config->DecimalAccuracy);
fprintf(fp, "Datum Index  %d\n", config->Datum);
fprintf(fp, "Gyro Index  %d\n", config->Gyro);
fprintf(fp, "Display Index  %d\n", config->DisplayType);
fprintf(fp, "Side Panel %d\n", config->SidePanelType);

// logging
fprintf(fp, "\nLog Disk Drive         %d\n", config->logDisk);
fprintf(fp, "Binary LoggingEnabled  %d\n", config->BinLoggingEnabled);
fprintf(fp, "Ascii Logging Enabled  %d\n", config->AsciiLogEnabled);
fprintf(fp, "Ascii Logging Interval  %d\n", config->AsciiLogInterval);

fprintf(fp, "Current Line  %s\n", config->curLine.name);
fprintf(fp, "   %.6lf", config->curLine.start.lat * RAD_TO_DEG);
fprintf(fp, "   %.6lf\n", config->curLine.start.lon * RAD_TO_DEG);
fprintf(fp, "   %.6lf", config->curLine.end.lat * RAD_TO_DEG);
fprintf(fp, "   %.6lf\n", config->curLine.end.lon * RAD_TO_DEG);
fprintf(fp, "Current Site  %s\n", config->curSite.name);
fprintf(fp, "   %.6lf", config->curSite.pos.lat * RAD_TO_DEG);
fprintf(fp, "   %.6lf\n", config->curSite.pos.lon * RAD_TO_DEG);
fprintf(fp, "Ship File  %s\n", config->ShipName);
fprintf(fp, "Coastline File  %s\n", config->CoastName);
fprintf(fp, "Binary Nav %s\n", config->binFilename);
fprintf(fp, " Ascii Nav %s\n", config->ascFilename);
fprintf(fp, "   Logbook %s\n", config->logFilename);
fprintf(fp, "Filter:   Time:%.0lf    Weight:%d\n", 
               config->filter.time, 
               config->filter.weight);
fprintf(fp, "Printer:   %s enabled to write every %d seconds\n", 
               (config->printer.enabled) ? "is" : "is Not", 
               config->printer.interval);
fprintf(fp, "     idx         = %d\n", config->printer.idx);
fprintf(fp, "     name        = %s\n", config->printer.name);
fprintf(fp, "     xScale      = %.2lf\n", config->printer.xScale);
fprintf(fp, "     yScale      = %.2lf\n", config->printer.yScale);

// plotter constants
fprintf(fp, "Plotter Interval: %d\n", config->plot.interval);
fprintf(fp, "Plotter destination: %d\n", config->plot.destination);
fprintf(fp, "Plotter sensor: %d\n", config->plot.sensor_idx);
fprintf(fp, "Plotter start: %.1lf\n", config->plot.start);
fprintf(fp, "Plotter stop: %.1lf\n", config->plot.stop);
fprintf(fp, "Plotter scale: %ld\n", config->plot.scale);
fprintf(fp, "Plotter center lat: %.8lf\n", config->plot.center.lat);
fprintf(fp, "Plotter center lon: %.8lf\n", config->plot.center.lon);
fprintf(fp, "Plotter file: %s\n", config->plot.filename);
fprintf(fp, "Plotter center_UTM X: %.2lf Y: %.2lf\n", 
                          config->plot.center_utm.x, config->plot.center_utm.y);
fprintf(fp, "Plotter center X: %5d Y: %5d\n", 
                          config->plot.HPcfg.center.X, config->plot.HPcfg.center.Y);
fprintf(fp, "Plotter p1 X: %5d Y: %5d\n", 
                          config->plot.HPcfg.p1.X, config->plot.HPcfg.p1.Y);
fprintf(fp, "Plotter p2 X: %5d Y: %5d\n", 
                          config->plot.HPcfg.p2.X, config->plot.HPcfg.p2.Y);
fprintf(fp, "Plotter xres: %.8lf\n", config->plot.xres);
fprintf(fp, "Plotter yres: %.8lf\n", config->plot.yres);

fprintf(fp, "Vrp:   dist = %.1lf   bearing = %.1lf\n", config->Vrp.dist, 
           RAD_TO_DEG * config->Vrp.bearing);

fprintf(fp, "utm:   %02d%c\n", config->utm.zoneNum, config->utm.zoneLtr);
fprintf(fp, "foreground:  %02d\n", config->fg);
fprintf(fp, "background:  %02d\n", config->bg);
fprintf(fp, "depth Correction:  %02d\n", config->depth.depthCorrectionEnabled);
fprintf(fp, "soundVelocity:  %.2lf\n", config->depth.soundVelocity);
fprintf(fp, "Displayed Sensors:\n");
for (i=0; i<MAX_COMPORTS; i++)
	{
	fprintf(fp, "%2d ", config->displaySensors[i]);
	}
fprintf(fp, "\n");

fprintf(fp, "\nRelays:\n");
fprintf(fp, "Relay Board is %s\n", (config->relay.enabled) ? "Enabled" : "Disabled");
fprintf(fp, "Base Address %X\n", config->relay.address);
fprintf(fp, "Board status %X\n", config->relay.status);
for (i=0; i<MAX_RELAYS; i++)
	{
	fprintf(fp, "Relay %d is %s  Interval %d lastClosure %ld\n", 
                     i, 
                     (config->relay.relaysEnabled[i]) ? "on " : "off",
                     config->relay.relayIntervals[i],
                     config->relay.lastTime[i]);
	}
fprintf(fp, "\n");

// video
fprintf(fp, "Video %s enabled\n", (config->videoEnabled) ? "is" : "is not");
fprintf(fp, "\n");

// miscellaneous
fprintf(fp, "shipCentering is %s\n", (config->misc.shipCentering) ? "On" : "Off");
fprintf(fp, "shipScaling is %s\n",   (config->misc.shipScaling) ? "On" : "Off");
fprintf(fp, "drawCoastline is %s\n", (config->misc.showCoastline) ? "On" : "Off");

fclose(fp);

// display a temp file name
DisplayAsciiFile( tmpname, 1, 32, 10, 50);

// get rid of temp file
remove(tmpname);

return SUCCESS;
}






/*----------------------------------------------------------------------------*/
// reset the colors
/*----------------------------------------------------------------------------*/
void resetColors(ob_desc * ob)
{
t_setup * config = QuerySetup();
fm_desc * parentFm;
int x, y;

if (QueryColors() <= 1)
  {
  SetAttrTblColors ( 0, 15 );
  }
else
  {
  SetAttrTblColors (config->fg, config->bg );
  }

parentFm = (fm_desc *) mainMenu->ob_lst[1].tag;
QryFmOrigin(parentFm, &x, &y);
CloseForm(parentFm);
OpenApp( NAV_VERSION , mainMenu, NULL );
OpenForm(parentFm, x, y);
	
// toggle object that caused this whole mess
ToggleRect ( &ob->area, rProt );

// Toggle the Edit label
ToggleRect (&mainMenu->ob_lst[1].area, rProt );
}

/*----------------------------------------------------------------------------*/
// do the foreground color form
/*----------------------------------------------------------------------------*/
int doColorFm(ob_desc * ob)
{
int retv;
fm_desc * fm;
char * frf = "color.frf";
t_setup * config = QuerySetup();
int * pColor;
int curColorIdx;
int fmIdx;
attr_blk * attr;
int i;

if (config == (t_setup *) NULL)
	{
	NotifyUser(-1, -1, NULL_SETUP_MSG, frf);
  return FALSE;
	}

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

if (stricmp(ob->label, "Foreground"))
	{
	// Must be background
	pColor = (int *) &config->bg;
	curColorIdx = config->bg % 16 + 2;
	strcpy(fm->title, "Background Color");
	fmIdx = 0;
	}
	else
	{
	// Must be foreground
	pColor = (int *) &config->fg;
	curColorIdx = config->fg % 16 + 2;
	strcpy(fm->title, "Foreground Color");
	fmIdx = 1;
	}

// set up form
fm->ob_lst [ curColorIdx ].flags |=  IS_CHECKED;

attr = GetAttrBlk ( attrCBut );
for (i=0; i<16; i++)
	{
	fm->ob_lst[i+2].attr = attrUsr1 + i;
	attr->fg_clr = i;
	PutAttrBlk ( attrUsr1+i, attr );
	}

OpenForm ( fm, ob->area.Xmin, ob->area.Ymin ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the fields
			*pColor = get_flag_fm_group(fm, 1);
			break;

		case CANCEL:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != CANCEL - 1);

CloseForm   ( fm );
DestroyFRFForm ( fm );

if (retv == OKAY)
	{
	resetColors(ob);
	}

return retv;
}



/*----------------------------------------------------------------------------*/
// return a pointer to the edit menu form.
/*----------------------------------------------------------------------------*/
fm_desc * queryEditFm(void)
{
return (fm_desc *) &mainMenu->ob_lst[1].tag;
}


/*----------------------------------------------------------------------------*/
// the main function
/*----------------------------------------------------------------------------*/
void main(int argc, char ** argv)
{
init_nav(argc, argv);
InitNavMenus();

while (1)
	ProcessMenu(mainMenu);
}

