typedef struct point_struct
	{
	int X;
	int Y;
	} t_point;

typedef struct depth_struct
  {
	int depthCorrectionEnabled;            // adjust the received depth ??
	double soundVelocity;                  // meters per second
	} t_depth;

typedef struct misc_struct
  {
	char shipCentering;            // keep the ship in the display ??
	char allowVideo;               // allow calls to live windows ??
	char shipScaling;              // draw ship to scale ??
	char showCoastline;            // draw coastline ??
	char fixSymbol;                // shape of fix symbol
	char future[9];                // room to grow
	} t_misc;

typedef struct grid_pos
  {
	double x;                     // UTM X coordinate
	double y;                     // UTM Y coordinate
	} t_grid_pos;

typedef struct fpos
  {
	float lat;                     // radians of latitude
	float lon;                     // radians of longitude
	} t_fpos;

typedef struct offset_struct
  {
	int id;                       // sensor index
	double x;                     // offset from centerline
	double y;                     // offset from amidships
	double z;                     // offset from main deck
	double dist;                  // vector length
	double bearing;               // vector angle
	} t_offset;

typedef struct utm_struct
  {
	int zoneNum;                    // utm zone number
	char zoneLtr;                   // Zone letter specifier
	} t_utm;

typedef struct pos
  {
	double lat;                     // radians of latitude
	double lon;                     // radians of longitude
	} t_pos;

typedef struct com_struct
  {
  char portname[6];             /* e.g COM10  */
  unsigned char portnum;        /* e.g. COM1 = 0  */
  unsigned char parity;         /* Parity  'O', 'N', or 'E' */
  unsigned char data;           /* Data Bits 7 or 8 */
  unsigned char stop;           /* Stop Bits  1 or 2 */
  unsigned int psd_word;        /* Parity, Stop and Data bit word */
  char sensor_name[30];         /* e.g Trimble 4000  */
  int sensor_index;             /* index to sensor list */
  unsigned int baud;            /* Baud Rate */
  unsigned char enabled;        // Is it on or off
  unsigned int inputQueue;      // bytes waiting to be processed
  int interval;                 // seconds between transmissions  
  unsigned int tail;            // pointer to the end of the buffer
	int handshake;                // type of handshake XON, CTS etc
  char buf[MAX_PORT_BUF_SIZE];  // buffer to store data in
  } t_com;


typedef struct station
	{
	char enabled;                 // is the station enabled for use
	char display;                 // is the station enabled for display
	int id;                       // transponder code
	char name[30];                // name of station
	t_pos pos;                    // location of station
	double height;                // height above ellipsoid of station in meters
	double bias;                  // What to add to the observed range in meters
	double delay;                 // Transponder turnaround delay in meters
	} t_station;
	

typedef struct calib_struct
	{
	int ids[MAX_NAV_STATIONS];        // transponder_id
	double ranges[MAX_NAV_STATIONS];  // Theoretical ranges meters
	double biases[MAX_NAV_STATIONS];  // C-O's in meters
	} t_calibration;


typedef struct pdd_struct
	{
	char alarm;                         // alarm state
	char dummy;                         // Keep structure aligned
	double rawDepth;                    // raw depth value from PDD
	double corDepth;                    // corrected depth value from PC
	double soundVel;                    // sound velocity in m/s
	} t_pdd;

typedef struct gps
	{
	char sat_buf[40];      // constellation in use
	int  sat_count;        // number of SV's used in fix
	double pdop;
	int  dgps;             // differential mode ??
	} t_gps;


typedef struct observation_struct
	{
	int numObs;                                // How many observations we have
	int iterations;                            // How many iterations it took
	double ranges[MAX_NAV_STATIONS];           // the observations 
	double bearings[MAX_NAV_STATIONS];         // the angle from pos to station 
	double residuals[MAX_NAV_STATIONS];        // the least squares residuals
	t_station * stations[MAX_NAV_STATIONS];    // the corresponding stations
	} t_observations;

typedef struct loran
	{
	int gri;                                // group repetition interval
	double tds[MAX_TDS];                    // time delays
	double emiss_delays[MAX_TDS];           // emission delays
	double coding_delays[MAX_TDS];          // coding delays
	double masterToa;                       // master time of arrival
	t_observations obs;
	} t_loran;


typedef struct trispo
	{
	t_observations obs;
	} t_trispo;

typedef struct usbl_struct
	{
	int transponder_id;                     // target #
	double compass_heading;                 // if connected degrees
	double horiz_angle;                     // rel bearing degrees
	double slant_range;                     // slant range meters
	double x;                               // relative x
	double y;                               // relative y
	double z;                               // relative z
	double telemetry;                       // depth meters
	int err_codes;                          // error codes
	} t_usbl;

typedef struct videoHook_struct
	{
	char fileName[15];                       // image filename
	int imageType;                           // 0=Native, 1=TIFF, 2=BMP
	int x1;                                  // bounding coordinate of image
  int y1;                                  // bounding coordinate of image
  int x2;                                  // bounding coordinate of image
  int y2;                                  // bounding coordinate of image
	} t_videoHook;

typedef struct sample_struct
	{
	struct sample_struct * previous;            // pointer to previous site
	int type;                                   // type of sample
	int display;                                // display the symbology ??
	char name[30];                              // site description
	t_pos pos;                                  // intended site location
	double depth;                               // actual depth of site
	char historyFile[15];                       // filename of history data
	t_videoHook vHook;                          // video image info
	struct sample_struct * next;                // pointer to next site
	} t_site;

typedef struct rngBrg_struct
	{
	int initialized;                // do we have the current station info
	t_site site;                 // site info
	double range;                  // meters from current pos to site
	double bearing;                // bearing from current pos to site
	} t_rngBrg;

typedef struct track_struct
	{
	int initialized;               // is the structure current ?
	char name[30];                 // name of the line being tracked
	double solX;                   // UTM X of the start of line
	double solY;                   // UTM Y of the start of line
	double eolX;                   // UTM X of the end of line
	double eolY;                   // UTM Y of the end of line
	double slope;                  // slope of the line
	double bearing;                // radians
	double length;                 // length in meters
	double cross;                  // meters of cross track distance
	double along;                  // meters of along track distance
	double toend;                  // meters to end of track distance
	} t_track;

typedef struct super_packet_struct
	{
	int navType;                   // line or station work ?
	t_track  line;               // line info
	t_rngBrg site;               // site info
	} t_super_packet;

typedef struct nav
	{
	unsigned char id;                 // ID byte
	unsigned char status;             // current sensor status byte
	double tim;                       // dos time 
	double lat;                       // radians
	double lon;                       // radians
	double alt;                       // meters of height above ellipsoid or depth
	double spd;                       // Knots
	double crs;                       // radians of course made good
	double hdg;                       // radians
	union  msmts                      // measurements
		{
		t_super_packet navPacket;       // everything needed to navigate
		t_usbl usbl;                    // Ultra Short Baseline Acoustics
		t_trispo rng;                   // ranging system 
		t_loran lor;                    // loran system
		t_gps gps;                      // GPS
		t_pdd pdd;                      // PDD Raytheon Bathy Digitizer
		} msmts;
	} t_nav;

typedef struct line
  {
	struct line * previous;          // pointer to previous line member
	char name[30];                   // line name
	int num;                         // integer line number
	int display;                     // Show or not show
	t_pos start;                     // starting position
	t_pos end;                       // ending position
	double range;                    // range in meters
	double bearing;                  // bearing in radians
	struct line * next;              // pointer to next line member
	} t_line;

typedef struct HPGL_struct
	{
	t_point p1;
	t_point p2;
	t_point center;
	double xRes;
	double yRes;
	} t_HPcfg;

typedef struct plot_struct
	{
	int destination;                // SCREEN = 0 Plotter = 1
	int interval;                   // Seconds between plotted values
	int sensor_idx;                 // Sensor to plot
	double start;                   // start time
	double stop;                    // stop time
	t_pos center;                   // plot center coordinates
	double xres;                    // resolution of plotter meters/gdu
	double yres;                    // resolution of plotter meters/gdu
	t_grid_pos center_utm;          // utm center
	long scale;                     // scale in meters 1:scale
	char filename[60];              // full path to plotfile
	char title[60];                 // plot title
	t_HPcfg HPcfg;                  // HPGL specifics
	t_com * port;                   // pointer to communications structure
	} t_plot;


typedef struct printer_struct
	{
	int enabled;                    // On or OFF
	int interval;                   // Seconds between updates
	int columns;                    // columns per page
	int lines;                      // lines per page
	double xScale;                  // Xscaling for screen dump
	double yScale;                  // Yscaling for screen dump
	int idx;                        // index into printer table
	char name[40];                  // name of indexed printer
	} t_printer;


typedef struct filter_struct
	{
	double time;                    // age of data in seconds
	int weight;                     // 0-100 weighting index for dynamics
	} t_filter;


typedef struct relay_struct              // Contact Closure Control
	{
	int enabled;
	int status;                            // board Status
	unsigned int address;                  // base address of board
	int relaysEnabled[MAX_RELAYS];         // enable relays
	int relayStates[MAX_RELAYS];           // high or low
	int relayIntervals[MAX_RELAYS];        // seconds between contact events
	int inputsEnabled[MAX_RELAYS];         // enable digital IO lines
	int inputStates[MAX_RELAYS];           // high or low
	int inputIntervals[MAX_RELAYS];        // seconds between input reads
	time_t lastTime[MAX_RELAYS];           // time of last relay cycle
	} t_relay;

typedef struct bridge_struct
	{
	int mode;
	} t_bridge;


typedef struct video_struct
	{
	int volume;
	int brightness;
	int color;
	int contrast;
	int hue;
	int source;
	int channel;
	} t_video;


typedef struct setup
	{
	int Initialized;       // Setup has been loaded
	int CoordSys;          // coordinate system LLA, UTM, State Plane
	int AngleType;         // radians, decimal degrees, deg min, deg min sec
	int HorizUnits;        // Meters, US feet, Int feet
	int DecimalAccuracy;   // places of decimal precision
	int Datum;             // Index to current Datum
	int Gyro;              // Sensor Index for Gyro Type 
	int DisplayType;       // Which display to start up in
	int logDisk;           // 0 = Current Drive, 1 = A, 2 = B etc
	int BinLoggingEnabled; // To log or not to log binary data
	int AsciiLogEnabled;   // To log or not to log  (curNav as Ascii Data)
	int AsciiLogInterval;  // Seconds between ascii logs
	t_line curLine;        // the last known survey line (always at least one)
	t_site curSite;        // the last known site (always at least one)
	char ShipName[15];     // Filename of ship shape
	char FishName[15];     // Filename of fish shape
	char CoastName[15];    // Filename of fish shape
	char binFilename[50];  // full path name of Nav data log
	char ascFilename[50];  // full path name of Ascii data file
	char logFilename[50];  // full path name of logbook file
	t_filter filter;       // filter configuration
	t_printer printer;     // printer configuration
	t_plot plot;           // plotter configuration
	t_offset Vrp;          // Vessel Reference Position (measured from stern)
	t_utm utm;             // utm parameters
	int fg;                // foreground color
	int bg;                // background color
	int displaySensors[MAX_COMPORTS];      // which sensors to track
	t_misc misc;           // miscellaneous display stuff
	t_depth depth;         // depth handling parameters
	int SidePanelType;     // Which display to use  for side panel
	t_bridge bridge;       // bridge terminal setup
	t_relay relay;         // relay setup structure
	int videoEnabled;      // Is the video board addressable
	} t_setup;

typedef struct ellipsoid
	{
	char name[20];									/* name of ellipsoid */
	double a;												/* semi-major axis */
	double b;												/* semi-minor axis */
	double f;												/* flattenning f=(a-b)/a  */
	double e2;											/* first eccentricity^2 */
	} t_ellips;

typedef struct datum
	{
	char id;                        // index to datum enumerated list
	char conversion;								// molodensky or multiple regression
	char name[30];									// name of datum 
	t_ellips ellipsoid;							// ellipsoid structure
	char coef_file[14];							// file containing datum shift cooeficients
	} t_datum;

typedef struct graf
	{
	long scale;                   // width/height of window in meters
	t_grid_pos center_utm;        // UTM coordinates of center of window
	double xres;                  // (window width pixels) / (scale meters)
	double yres;                  // (window height pixels) / (scale meters)
	int centerX;                  // center of window in pixels
	int centerY;                  // center of window in pixels
	} t_graf;

typedef struct vector_struct
	{
	double magnitude;                 // vector length or magnitude
	double direction;              // vector bearing or direction
	} t_vector;

typedef struct graphic_display_struct
	{
	int type;                              // entity type, e.g. point, line oval
	int on;                                // is it on or off
	int fillAble;                          // is it fillable
	int frameAble;                         // is it frameable
	int fillColor;                         // the fill color
	int frameColor;                        // the frame color
	int fillPattern;                       // metagraphics pattern index
	int lineType;                          // metagraphics pen dash
	int penSize;                           // metagraphics pen size
	int penColor;                          // metagraphics pen Color
	} t_gDisplay;

typedef struct graphic_point_struct
	{
	t_gDisplay display;                    // how to draw one of these
	t_vector   v;                          // where to draw one of these
	} t_gPoint;

typedef struct graphic_line_struct
	{
	t_gDisplay display;                    // how to draw one of these
	t_vector   v1;                         // where to start drawing
	t_vector   v2;                         // where to stop drawing
	} t_gLine;

typedef struct graphic_oval_struct
	{
	t_gDisplay display;                    // how to draw one of these
	t_vector   v;                          // where to draw one of these
	double xRadius;                        // size 
	double yRadius;                        // size 
	} t_gOval;

typedef struct graphic_entity_struct
	{
	int type;                              // what this entity is
	union                                  // the universe of possible entities
		{
		t_gPoint  point;                     // points
		t_gLine   line;                      // lines
		t_gOval oval;                        // ovals
		} shape;
	} t_gEntity;

typedef struct shape_struct
	{
	int numEntities;                        // number of graphic entities used
	char name[30];                          // name of the graphic construct
	t_gEntity entities[MAX_ENTITIES];       // array of vectors defining shape
	} t_shape;

typedef struct fix_struct
	{
	double tim;                   // time of fix
	t_pos pos;                    // position of fix
	} t_fix;

typedef struct lineFix_struct
	{
	double cross;                    // cross track error in meters
	double along;                    // along track error in meters
	} t_lineFix;

typedef struct history_struct
	{
	int numPts;                    // number of current elements 
	t_fix fix[HISTORY_SIZE];       // array of position fixes
	} t_history;

typedef struct bridge_history_struct
	{
	int numPts;                    // number of current elements 
	t_lineFix fix[BRIDGE_HISTORY_SIZE];   // array of line fixes
	} t_bridgeHistory;

typedef struct coast_struct
	{
	int numPts;                    // number of current elements 
	t_fpos pts[MAX_COASTLINE_SIZE]; // array of positions 
	} t_coast;



typedef struct bridgeMsg_struct          // Packet id 0x61
	{
	int urgency;                           // level of urgency/action
	char msg[80];                          // Null terminated msg
	} t_bridgeMsg;


typedef struct bridgeSetup_struct        // Packet id 0x60
	{
	int mode;                              // Line or site mode
	} t_bridgeSetup;


typedef struct vessel_config_struct
	{
	int initialized;       // Setup has been loaded
	int gridOn;            // is Grid On?
	int gridDx;            // grid X spacing in pixels
	int gridDy;            // grid Y spacing in pixels
	int snapOn;            // Is Snap On?
	int scale;             // scale of drawing
	int units;             // 0=METERS, 1=FEET. 2=YARDS
	int fg;                // foreground color
	int bg;                // background color
	t_printer printer;     // printer defaults
	t_gDisplay display;    // graphic display attribute defaults
	} t_vesselCfg;
