// printer.c
// functions for controlling printer
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <conio.h>
#include <ctype.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>
#include <bios.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include <bd_def.h>
#include <bd_pro.h>

#include "navdefs.h"
#include "navtypes.h"
#include "nav_util.h"
#include "printer.h"         // prototypes
#include "filehndl.h"        // prototypes
#include "mnutils.h"        // prototypes
#include "graph.h"        // prototypes

#define PRSTATUS(port)	  (_bios_printer (_PRINTER_STATUS, (port), 0))
#define PR_S_NOTBUSY      0x80
#define PR_S_ONLINE       0x10
#define PR_OK             (PR_S_ONLINE | PR_S_NOTBUSY)
#define PRINTER 6

/* Global data */
extern t_com com_struct[];
extern t_nav navSensors[];
extern t_nav * curNav;
extern char DisplayingSomething;


static char * SensorNames[] = 
	{
	"Magnavox MX1107\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"Ashtech GPS\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"Del Norte DDMU 540\0\0\0\0\0\0\0\0\0\0\0\0",
	"Northstar 7000 Loran C\0\0\0\0\0\0\0\0",
	"Trimble 4000 AX\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"O.R.E. Trackpoint II\0\0\0\0\0\0\0\0\0\0",
	"Bridge Terminal IO\0\0\0\0\0\0\0\0\0\0\0\0",
	"DataSonics Nav\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"KVH Gyro\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"DelNorte Out to URI System\0\0\0\0",
	"Trackpoint Out to URI System\0\0",
	"Buster Gyro Digitizer\0\0\0\0\0\0\0\0\0",
	"Trimble NavTrakII\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"Trimble 10X\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"Mudshark\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0",
	"Raytheon PDD Depth Digitizer\0\0",
	"HP Compatible Plotter\0\0\0\0\0\0\0\0\0"
 	};
static char * CANT_LOAD_MSG   = "Can't Load";
static char * NULL_SETUP_MSG  = "Null Configuration";
static char printerIsOpen = FALSE;
static fm_desc * indicator = (fm_desc *) NULL;


/*----------------------------------------------------------------------------*/
// Check if printer is on line
/*----------------------------------------------------------------------------*/
int checkPrinter(void)
{
int retval =  PRSTATUS(0) != PR_OK;
return (retval != PR_OK) ? retval : SUCCESS;
}


/*----------------------------------------------------------------------------*/
// Print current Nav if time to do so
/*----------------------------------------------------------------------------*/
void printNav(void)
{
static time_t lastTime = 0L;
t_setup * config = QuerySetup();
time_t now = time(NULL);
char latBuf[40];
char lonBuf[40];
char buf[40];
t_track * track = queryTrackInfo();
t_rngBrg * site;

if ((site = querySiteInfo()) == (t_rngBrg *) NULL)
	{
	logMsg("Null Site\n");
	return;
	}

if (checkPrinter() != SUCCESS)
	return;

if ((config->printer.enabled) &&
    (now - lastTime >= (time_t) config->printer.interval) &&
    ((now % (time_t) config->printer.interval) == 0L) )
	{
	lastTime = now;

	PositionString(lonBuf, latBuf, buf, 
                 curNav->lon, curNav->lat, curNav->alt, 0);
	if (config->SidePanelType == LINE_INFO)
		{
		fprintf(stdprn, "%s %s %s %s %s\n    %s Cross: %.1lf Along: %.1lf ToEnd: %.1lf\n   Speed: %.1lf  Gyro: %.1lf  Course: %.1lf\n\n", getTimeStr(0.0), latBuf, lonBuf, buf, SensorNames[curNav->id], config->curLine.name, track->cross, track->along, track->toend, curNav->spd, curNav->hdg * RAD_TO_DEG, curNav->crs * RAD_TO_DEG);
		}
	else
		{
		fprintf(stdprn, "%s %s %s %s %s\n    %s Range: %.1lf Bearing: %.1lf\n   Speed: %.1lf  Gyro: %.1lf  Course: %.1lf\n\n", getTimeStr(0.0), latBuf, lonBuf, buf, SensorNames[curNav->id], config->curSite.name, site->range, site->bearing, curNav->spd, curNav->hdg * RAD_TO_DEG, curNav->crs * RAD_TO_DEG);
		}
	}

}



/*----------------------------------------------------------------------------*/
// Print Message
/*----------------------------------------------------------------------------*/
void printMsg(char * msg)
{
if (msg == (char *) NULL)
	return;

if (checkPrinter() != SUCCESS)
	return;

fprintf(stdprn, "%s: %s\n", getTimeStr(0.0), msg);
}


/*----------------------------------------------------------------------------*/
// Mark position
/*----------------------------------------------------------------------------*/
void doMarkPosition(void)
{
t_setup * config = QuerySetup();
char latBuf[40];
char lonBuf[40];
char altBuf[256];
char buf[256];
char * asterisks = "************************* Mark Position *****************";

PositionString(lonBuf, latBuf, altBuf, 
                 curNav->lon, curNav->lat, 0.0, 0);

sprintf(buf, "\n%s\n%s %s %s\n%s",asterisks, latBuf, lonBuf, altBuf, asterisks);

printMsg(buf);
logMsg(buf);
}


/*----------------------------------------------------------------------------*/
// Print a rectangle
/*----------------------------------------------------------------------------*/
int printRect(rect * r)
{
return prt_win(r->Xmin, r->Ymin, r->Xmax, r->Ymax);   
}


/*----------------------------------------------------------------------------*/
/* Report the status of Baby Driver Printing                                  */
/*----------------------------------------------------------------------------*/
int far print_screen_status(void)
{
return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// Check for a print screen request                                   
/*----------------------------------------------------------------------------*/
int check_for_print_screen(app_msg * key)
{
char prtDestination[_MAX_PATH];
rect * r;

if (( isCTL(key->msgFlags)) && 
       /* CTL-down arrow, or CTL-P */
       ((key->msgValue == 0x50) || (key->msgValue == 0x70)))
	{
	DisplayingSomething = TRUE;
	if (!QueryUser(-1, -1, "NOTE", "Do You Want To Print This Screen?"))
		{
		if (OKAY == getPrintDestination(prtDestination))
			{
			if (openPrinter(prtDestination) == SUCCESS)
				{
				pset_cksts(screenDumpStatus);
				r = queryPrintRect();
				buildIndicator(r->Xmax + 10, -1);
				HideCursor();
				printRect(r);
				ShowCursor();
				pg_eject();
				killIndicator();
				}
			else
				{
				NotifyUser(-1, -1, "Yo", "Unable to reset printer");
				return USER_CANCEL;
				}
			}  // end of if getPrtDestination ...
		DisplayingSomething = FALSE;
		return TRUE;
		}  /* end of if query_user() */
	}  /* end of if isALT(.... */

DisplayingSomething = FALSE;
return FALSE;
}






/*----------------------------------------------------------------------------*/
// Get the destination of the preint screen output
// First ask for file or hardcopy
// If hardcopy return LPT1
// If file use file box to get input file name
/*----------------------------------------------------------------------------*/
int getPrintDestination(char * prtDest)
{
int retv;
fm_desc * fm;
char * frf = "prt_dest.frf";
char filename[_MAX_PATH];
char drive[_MAX_DRIVE];
char dir[_MAX_DIR];
char fname[_MAX_FNAME];
char ext[_MAX_EXT];
int driveNum=0;
t_setup * config = QuerySetup();

/* Load the FRF file */
if ( ( fm = LoadFRF ( frf ) ) == NULL ) 
	{
	NotifyUser(-1, -1, CANT_LOAD_MSG, frf);
  return FALSE;
	}

DisplayingSomething = TRUE;
OpenForm ( fm, -1, -1);

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case HARDCOPY:
			strcpy(prtDest, "LPT1");
			break;

		case DUMP_TO_FILE:
			// get the output file name
			_splitpath(config->ascFilename, drive, dir, fname, ext);
			_makepath(filename, drive, dir, fname, ".prt");

			if ( (!get_output_file(filename, "Dump Screen To", filename) ))
				{
				retv = NOT_SET;
				}
			else
				{
				strcpy(prtDest, filename);
				}
			break;
		}
	} while (retv != HARDCOPY && retv != DUMP_TO_FILE && retv != NOT_SET);

CloseForm   ( fm );
DestroyFRFForm ( fm );

DisplayingSomething = FALSE;
return (retv == NOT_SET) ? NOT_SET : OKAY;
}





/*----------------------------------------------------------------------------*/
// Open the printer device
/*----------------------------------------------------------------------------*/
int openPrinter(char * destination)
{
t_setup * config = QuerySetup();
char buf[128];

int x_scale = (int) (config->printer.xScale * 100.0);
int y_scale = (int) (config->printer.yScale * 100.0);

if (printerIsOpen)
	{
	closePrinter();
	}

pr_select(config->printer.idx+1);

if (pr_open(destination) == OKAY)
	{
	// convert if necessary to match printer's image type 
	if (_pd.img_type != iRGB)
		{
		switch( _pd.img_type )
			{
			case iGRAY :  
				pal_cvt( pGRAY );  
				break;

			case iMONO :  
				pal_cvt( pMONO );  
				break;

			case iYMC  :  
				pal_cvt( pYMC );   
				break;

			case iYMCK :  
				pal_cvt( pYMCK );  
				break;
			}
		}
	pset_pxl( GetPixel );     
	pset_rot(iROT270);
	pset_res(rHIGH);
	pset_scl(x_scale, y_scale);
	return SUCCESS;	
	}
	
sprintf(buf, "Can't open printer as %s: %s\n", destination, perr_msg());
logMsg(buf);

return BAD_PRINTER_OPEN;
}






/*----------------------------------------------------------------------------*/
// Select a Printer from the list of Baby Driver supported printers           
/*----------------------------------------------------------------------------*/
int selectPrinter( int * idx , char pr_buf[])
{
fm_desc *lf;
char **names;
int cur_slot, top_slot, retv;
int nprinters = 0;
int i;

/* load the printer names */
// get the number of printers in the pdt file
nprinters = pdt_pcnt();

if ( !nprinters ) 
	{
	NotifyUser(-1, -1, "ERROR", "No Printer Names Allocated." );
	return -1;
	}
	
// allocate the pointers
names = (char **) malloc(nprinters * sizeof(char *));
if (names == (char **) NULL)
	{
	printf("%s:%d: %s\n", __FILE__, __LINE__, perr_msg());
	return  -2;
	}

// allocate memory for each pointer
for (i=0; i< nprinters; i++)
	{
	names[i] = (char *) malloc(32);
	memset(names[i], 0, 32);
	}

// fill in the names
if (pdt_names(names) != OKAY)
	{
	printf("%s:%d: %s\n", __FILE__, __LINE__, perr_msg());
	return  -2;
	}

/* create a list form */
lf = CreateListForm ( "Select a Printer", 7, 4, 34, 8, 0 );

lf->ob_lst [  9 ].label  = "Okay";
lf->ob_lst [  9 ].flags |= IS_EXITABLE;
lf->ob_lst [  9 ].key    = 'O';
lf->ob_lst [  9 ].state  = 8;
lf->ob_lst [ 10 ].label  = "Default";
lf->ob_lst [ 10 ].flags |= IS_EXITABLE;
lf->ob_lst [ 10 ].key    = 'D';
lf->ob_lst [ 10 ].state  = 8;
   
lf->ob_lst [ 11 ].label  = "Cancel";
lf->ob_lst [ 11 ].flags |= IS_EXITABLE;
lf->ob_lst [ 11 ].key    = 'C';
lf->ob_lst [ 11 ].state  = 8;
   
lf->ob_lst [ 12 ].label  = "Help";
lf->ob_lst [ 12 ].flags |= IS_EXITABLE;
lf->ob_lst [ 12 ].key    = 'H';
lf->ob_lst [ 12 ].state  = 8;

cur_slot = top_slot = (*idx < 0 || *idx >= nprinters) ? 0 : *idx;

/* open a form to put the list on */
OpenForm ( lf, 10, -1 );

do 
	{
	retv = ProcessListForm(lf, names, nprinters, &top_slot, &cur_slot );
	switch ( retv ) 
		{
		case 9 :                // Okay
			*idx = cur_slot;
			strcpy(pr_buf, names[cur_slot]);
			break;
      
		case 10:                // Default
			cur_slot = top_slot = (*idx < 0 || *idx >= nprinters) ? 0 : *idx;
			DrawForm ( lf );
			break;
		}

	} while ( ( retv != 9 ) && ( retv != 11 ) );

i = nprinters;
while ( i ) free ( names [ --i ] );

CloseForm   ( lf );
DestroyForm ( lf );

if (retv == 9)
	{
	return cur_slot;
	}
else
	{
	return -1;
	}
}




/*----------------------------------------------------------------------------*/
// Handle the printer setup
/*----------------------------------------------------------------------------*/
int doPrinterFm ( ob_desc * ob )
{
int retv;
char pr_buf[40];             /* Place to put printer name */
int  pr_idx;                 /* Index into printer table */
int result;
fm_desc *fm;
ftag * colTag;
ftag * linTag;
ftag * prxTag;
ftag * pryTag;
ftag * namTag;
ftag * intervalTag;
char * frf = "printer.frf";
t_setup * config = QuerySetup();

 // Load the FRF file 
if ( ( fm = LoadFRF ( frf ) ) == NULL ) 
	{
	NotifyUser(ob->area.Xmax, ob->area.Ymin, CANT_LOAD_MSG, frf);
  return FALSE;
	}

colTag = (ftag *) fm->ob_lst[2].tag;
linTag = (ftag *) fm->ob_lst[3].tag;
prxTag = (ftag *) fm->ob_lst[4].tag;
pryTag = (ftag *) fm->ob_lst[5].tag;
namTag = (ftag *) fm->ob_lst[6].tag;
intervalTag = (ftag *) fm->ob_lst[8].tag;

sprintf(colTag->fld, "%d",    config->printer.columns);
sprintf(linTag->fld, "%d",    config->printer.lines);
sprintf(prxTag->fld, "%.1lf", config->printer.xScale);
sprintf(pryTag->fld, "%.1lf", config->printer.yScale);
strcpy(namTag->fld,           config->printer.name);
sprintf(intervalTag->fld, "%d",  config->printer.interval / 60);

if (config->printer.enabled)
	{
	fm->ob_lst [ 7 ].flags |=  IS_CHECKED;
	}
else
	{
	fm->ob_lst [ 7 ].flags &= ~IS_CHECKED;
	}


pr_idx =  config->printer.idx;

OpenForm ( fm, ob->area.Xmin, ob->area.Ymin ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY: 
			if (fm->ob_lst[7].flags & IS_CHECKED)
				{
				if (checkPrinter() == SUCCESS)
					{
					config->printer.enabled = TRUE;
					}
				else
					{
					NotifyUser(-1, -1, "Yo", "Printer is Offline");
					config->printer.enabled = FALSE;
					}
				}
			result = atoi(intervalTag->fld);
			if (result < 0)
				config->printer.interval = 60;               // default is 1 minute
			else
				config->printer.interval = result * 60;

			config->printer.idx     = pr_idx;
			config->printer.columns = atoi(colTag->fld);
			config->printer.lines   = atoi(linTag->fld);
			config->printer.xScale  = atof(prxTag->fld);
			config->printer.yScale  = atof(pryTag->fld);
			strcpy(config->printer.name, namTag->fld);
			if (config->printer.enabled)
				openPrinter("LPT1");
			break;

		case CANCEL-1: 
			break;

		case PRINTER : 
			Notify(-1, -1, "Scanning Printer Table...");
			result = selectPrinter(&pr_idx, pr_buf);
			NotifyOff();
			if (result >= 0)
				{
				strcpy(namTag->fld, pr_buf);
				DrawTaggedOb(&(fm->ob_lst[6]));
				}
			break;
		}
	} while ( ( retv != OKAY ) && ( retv != CANCEL-1 ) );

CloseForm      ( fm );
DestroyFRFForm ( fm );
}



/*----------------------------------------------------------------------------*/
// close the printer and restore the static variable
/*----------------------------------------------------------------------------*/
void closePrinter(void)
{
pr_close();
printerIsOpen = FALSE;
}



/*----------------------------------------------------------------------------*/
// Report the status of Baby Driver Printing                                  */
/*----------------------------------------------------------------------------*/
int far screenDumpStatus(void)
{
char buf[40];
long pct;
app_msg * aM;

pct = (long) ((long) _ps.sw_cur * 100L) / (long) _ps.sw_cnt;
UpdateIndicator(indicator, (int) pct);

if (MsgInQueue())
	{
	aM = ReadMsgQueue();
	if (isEscapeKey(aM) || isEnterKey(aM) || isMsPress(aM))
		{
		FlushMsgQueue();
		ShowCursor();
		sprintf(buf, "Printing.... %ld%% complete", pct);
		if (QueryUser(-1, -1, buf, "Continue Printing ?"))
			{
			FlushMsgQueue();
			killIndicator();
			return FAILED;
			}
		HideCursor();
		FlushMsgQueue();
		}
	FlushMsgQueue();
	}

return OKAY;
}



/*----------------------------------------------------------------------------*/
// build the indicator guage at the x and y coords
/*----------------------------------------------------------------------------*/
fm_desc * buildIndicator(int x, int y)
{
// see if it is already open
if (indicator != (fm_desc *) NULL)
	return (fm_desc *) NULL;

return indicator = OpenIndicator(x, y, "Printing...", "50:0:100");
}




/*----------------------------------------------------------------------------*/
// Close and kill the indicator
/*----------------------------------------------------------------------------*/
void killIndicator(void)
{
if (indicator == (fm_desc *) NULL)
	return;

CloseIndicator(indicator);
indicator = (fm_desc *) NULL;
}

