#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <conio.h>
#include <time.h>

#include "navdefs.h"
#include "navtypes.h"
#include "nav_Util.h"
#include "readlog.h"
t_nav nav;

static unsigned char dumpFlags = 0x00;	


char * SensorNames[] = 
	{
  "ABC GYRO",
  "ASHTECH",
  "DDMU 540",
  "NS7000",
  "TRIMBLE 4000",
  "TRACKPOINT II",
  "BRIDGE TERM",
  "SIMRAD",
  "KVH GYRO",
  "DelNorte to URI",
  "Trackpoint to URI",
  "SEAHAWK GYRO",
  "TRIMBLE_NAVTRAK",
  "TRIMBLE_10X",
	"Mudshark",
	"HP Plotter"
	};


void main(int argc, char ** argv)
{
printf("Current Nav size %d\n", sizeof(t_nav));

process_cmd_line_flags(argc, argv);

if (argc < 2)
	{
	printf("Usage: %s <filename>\n", argv[0]);
	exit(1);
	}

readLog(argv[1]);
}


/*----------------------------------------------------------------------------*/
/* read the nav setup
/*----------------------------------------------------------------------------*/
int readLog(char * filename)
{
FILE *fp;
int k;
char buf[1024];

if ((fp = fopen(filename, "rb")) == NULL)
  {
  printf("\nCan't open file %s", filename);
  return FALSE;
  }

while (!feof(fp))
	{
	k = fread( (char *) &nav, sizeof(t_nav), 1, fp);
	if (k == -1)
		{
		fclose(fp);
		printf("\nfread failed");
		return FALSE;
		}

 	if ((nav.id & dumpFlags || 
     (dumpFlags == 0 && nav.id == 0) || 
     dumpFlags == 0xFF) && 
     (!feof(fp)))
		printf("%s\n", dumpNav(&nav, buf));
	}

fclose(fp);
return TRUE;
}







/*----------------------------------------------------------------------------*/
// dump the nav into an ascii buffer
/*----------------------------------------------------------------------------*/
char * dumpNav(t_nav * p, char * buf)
{
int j = 0;
char latBuf[40];
char lonBuf[40];
char altBuf[40];

j += sprintf(buf+j, "%s ", getTimeStr((double) p->tim));

if (p->id < sizeof(SensorNames) / sizeof(SensorNames[0]))
	j += sprintf(buf+j, "%s ", SensorNames[p->id]);
else
	j += sprintf(buf+j, "Unknown (%d) ", p->id);

PositionString(lonBuf, latBuf, altBuf, p->lon, p->lat, p->alt, 0);
j += sprintf(buf+j, "%s %s %s ", latBuf, lonBuf, altBuf);

j += sprintf(buf+j, "%.1lf ", MPS_TO_KTS * p->spd);
j += sprintf(buf+j, "%.1lf ", RAD_TO_DEG * p->crs);
j += sprintf(buf+j, "%.1lf ", RAD_TO_DEG * p->hdg);
switch(p->id)
	{
	case BRIDGE_TERM:
		dumpBridge(p, buf+j);
		break;

	default:
		break;
	}

return buf;
}




/*------------------------------------------------------------------*/
// Process command line flags 
/*------------------------------------------------------------------*/
void process_cmd_line_flags(int argc, char ** argv)
{
int i;

for (i=1; i<argc; i++)
	{
	if ( (!strnicmp(argv[i], "-gps", 4)) ||
       (!strnicmp(argv[i], "/gps", 4)) )
		{
		dumpFlags |= TRIMBLE_4000;
		}

	if ( (!strnicmp(argv[i], "-gyro", 4)) ||
       (!strnicmp(argv[i], "/gyro", 4)) )
		{
		dumpFlags |= MAGNAVOX_1107;
		}

	if ( (!strnicmp(argv[i], "-usbl", 5)) ||
       (!strnicmp(argv[i], "/usbl", 5)) )
		{
		dumpFlags |= TRACKPOINT_II;
		}

	if ( (!strnicmp(argv[i], "-bridge", 5)) ||
       (!strnicmp(argv[i], "/bridge", 5)) )
		{
		dumpFlags |= BRIDGE_TERM;
		}

	if ( (!strnicmp(argv[i], "-rng", 4)) ||
       (!strnicmp(argv[i], "/rng", 4)) )
		{
		dumpFlags |= DDMU540;
		}

	if ( (!strnicmp(argv[i], "-all", 4)) ||
       (!strnicmp(argv[i], "/all", 4)) )
		{
		dumpFlags = 0xFF;
		}
	}
}




/*----------------------------------------------------------------------------*/
// dump the track info
/*----------------------------------------------------------------------------*/
void dumpBridge(t_nav * p, char * buf)
{
int j = 0;
t_track * t = &p->msmts.navPacket.line;

j += sprintf(buf+j, "name = %s\n", t->name);
j += sprintf(buf+j, "bearing = %.2lf\n", t->bearing * RAD_TO_DEG);
j += sprintf(buf+j, "cross = %.2lf\n", t->cross);
j += sprintf(buf+j, "along = %.2lf\n", t->along);
j += sprintf(buf+j, "end   = %.2lf\n", t->toend  );
}



