// reduce.c
// John T. Gann
// Start Date 14 March 1992
// Least Squares reduction functions for range range position solution
// Uses non-weighted iterative least squares adjustment to solve 2- 10 ranges
//
 
#include <stdio.h>
#include <conio.h>
#include <process.h>
#include <dos.h>
#include <graph.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <malloc.h>
#include <time.h>
#include <sys/timeb.h>

#include "navdefs.h"
#include "navtypes.h"
#include "utmlib.h"
#include "nav_util.h"
#include "reduce.h"           // Prototypes

/*----------------------------------------------------------------------------*/
// Least Squares adjustment
/*----------------------------------------------------------------------------*/
int	reduceObservations(t_observations * obs, double * latfix, double * lonfix)
{
double distance[MAX_NAV_STATIONS];
double obs_matrix[MAX_NAV_STATIONS][3];   // observation matrix
double norm_matrix[2][3];	                // normal matrix 
double delta_lat;                         // iterative change in minutes of arc
double delta_lon;                         // iterative change in minutes of arc
double delta_latrad;                      // iterative change in radians
double delta_lonrad;                      // iterative change in radians
double det;                               // determinant of normal matrix
double fazm;                              // forward azimuth
double bazm;                              // back azimuth
double change = 999999.9;                 // iterative chg in position meters
register int i;                           // loop counter optimized
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps
double dx, dy;                            // components of radial distance

// See if there are enough ranges to compute a fix
if (obs->numObs < 2)
	return NOT_ENOUGH_RANGES;

// initialize the observation matrix  
memset(obs_matrix, 0x00, sizeof(obs_matrix));
memset(norm_matrix, 0x00, sizeof(norm_matrix));
obs->iterations = 0;

// iterate until the iterative change is small or too many iterations
while ((change > 1.0) && (obs->iterations++ < MAX_LSQ_ITERATIONS))  
	{
	// compute distance & azimuth to stations	from passed position 
	for (i=0; i<obs->numObs; i++) 
		{
		arclen(*latfix, *lonfix,
           obs->stations[i]->pos.lat,
           obs->stations[i]->pos.lon,
           &distance[i],
           &fazm,
           &bazm,
           ellipse->a,
           ellipse->b);
		obs->bearings[i] = fazm;
		obs_matrix[i][0] = cos(obs->bearings[i]);
		obs_matrix[i][1] = sin(obs->bearings[i]) * cos(*latfix);
		obs_matrix[i][2] = (distance[i] - obs->ranges[i]) / 1852.0;
		}

	for (i=0; i<obs->numObs; i++) 
		{
		norm_matrix[0][0] += obs_matrix[i][0] * obs_matrix[i][0];
		norm_matrix[1][1] += obs_matrix[i][1] * obs_matrix[i][1];
		norm_matrix[1][0] += obs_matrix[i][0] * obs_matrix[i][1];
		norm_matrix[0][1]  = norm_matrix[1][0];
		norm_matrix[0][2] += obs_matrix[i][0] * obs_matrix[i][2];
		norm_matrix[1][2] += obs_matrix[i][1] * obs_matrix[i][2];
		}

	// Compute the determinant of the normal matrix
	det = norm_matrix[0][0] * norm_matrix[1][1]
      - norm_matrix[0][1] * norm_matrix[1][0];

	// compute the change in latitude
	delta_lat = 1.0 / det * 
              (norm_matrix[1][1] * norm_matrix[0][2]
             - norm_matrix[0][1] * norm_matrix[1][2]);

	// compute the change in longitude
	delta_lon = 1.0 / det * 
              (-norm_matrix[1][0] * norm_matrix[0][2]
             + norm_matrix[0][0] * norm_matrix[1][2]);

	delta_latrad = delta_lat / 60.0 * DEG_TO_RAD;
	delta_lonrad = delta_lon / 60.0 * DEG_TO_RAD;

	// compute the plane distance of the delta components
	dy = delta_latrad * length_of_radian_of_lat(ellipse->a, ellipse->e2, *latfix);
	dx = delta_lonrad * length_of_radian_of_lon(ellipse->a, ellipse->e2, *lonfix);

	// the radial distance of the change
	change = hypot(dx, dy);

	// update the assumed position
	*latfix += delta_latrad;
	*lonfix += delta_lonrad;

	// sanity checks
	if (*lonfix < -PI)
		*lonfix += TWOPI;

	if (*lonfix >  PI)
		*lonfix += -TWOPI;

	} // end of while loop

// update the residuals
for (i=0; i<obs->numObs; i++)
	obs->residuals[i] = obs_matrix[i][2] * 1852.0;

return SUCCESS;
}

