// serial.c
// low level functions for controlling Arnet Smartport 8
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <conio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "nav_util.h"
#include "mnutils.h"
#include "utmlib.h"
#include "navtypes.h"
#include "graph.h"
#include "stations.h"
#include "reduce.h"
#include "serial.h"
#include "fileio.h"
#include "offsets.h"
#include "nav.h"
#include "ec_comm.h"
#include "relay.h"
#include "stans.h"
#include "tansdefs.h"
#include "tanstype.h"


#define ACCEPT 2

// private data
static char *com_names[] = {
	"COM1 ", "COM2 ", "COM3 ", "COM4 ", "COM5 ", 
  "COM6 ", "COM7 ", "COM8 ", "COM9 ", "COM10"
   };

/* Global data */
char p_chars[] = {'N', 'O', 'E'};
char d_chars[] = {'7', '8'};
char s_chars[] = {'1', '2'};

extern char _Debug;
extern t_com com_struct[];
extern t_nav navSensors[];
extern t_nav * curNav;
extern fm_desc * navDispFm;
extern char DisplayingSomething;
extern char _SmartPortInstalled;
extern int depthAlarmFlag;
static char CalibrateOn;
static t_calibration Calib;

static fm_desc * calibFm = (fm_desc *) NULL;
static char * NULL_NAV_MSG = "Null NAV pointer";
static unsigned int (*inputQueueTask)(int) = iq_check;
static int (*getCharTask)(int) = getArnetChar;
static int (*putCharTask)(int, unsigned char) = putArnetChar;

/*----------------------------------------------------------------------------*/
// Set Buad, Parity, Data and Stop Bits on Port
/*----------------------------------------------------------------------------*/
int set_baud(unsigned port, unsigned baud, unsigned char parity, 
                            unsigned char data_bits, 
                            unsigned char stop_bits,
                            int handshake)
{
union REGS inregs, outregs;
unsigned char parity_byte = build_parity_byte(parity, data_bits, stop_bits);

if (_SmartPortInstalled)
	{
	inregs.h.ah = 4;                  // set rs232 protocol
	inregs.h.al = parity_byte;
	inregs.x.bx = baud;
	inregs.x.dx = port;
	int86(RSINT, &inregs, &outregs);
		
	if (outregs.h.ah)
		{
		return 1;
		}
		else
		{
		// Set up handshake
		if (set_handshake(port, handshake) != SUCCESS)
			printf("Unable to set handshaking on COM%d\n", port + 1);
	
		// time_out(port);
		}
	}
else
	{
	ecSetBaud(port, baud, parity, data_bits, stop_bits);
	}
		
	
return 0;
}



/*----------------------------------------------------------------------------*/
// Set timeout to something reasonable
/*----------------------------------------------------------------------------*/
void time_out(unsigned int portnum)
{
union REGS inregs, outregs;
struct SREGS segregs;
inregs.h.ah = FUNC_19;
inregs.h.al = 0;                 // 2 ticks is about 1/9 sec
inregs.x.bx = 2;
inregs.x.dx = (unsigned int) portnum;

int86x(RSINT, &inregs, &outregs, &segregs);

return;
}



/*----------------------------------------------------------------------------*/
// Create the byte for the AL register for parity data and stop bits
/*----------------------------------------------------------------------------*/
unsigned char build_parity_byte(unsigned char parity,      // 'N', 'O', 'E'
                                unsigned char data_bits,   //  7, or 8
                                unsigned char stop_bits)   //  1, or 0
{
unsigned char parity_byte = 0x00;

switch (parity)                       // bits 4-3 (2^4 - 2^3)
	{
	case 'N':
		parity_byte |= 0x00;
		break;

	case 'O':
		parity_byte |= 0x08;
		break;

	case 'E':
		parity_byte |= 0x18;
		break;
	}

switch (data_bits)                         // bits 0-1 (2^0 - 2^1)
	{
	case 5:
		parity_byte |= 0x00;
		break;

	case 6:
		parity_byte |= 0x01;
		break;

	case 7:
		parity_byte |= 0x02;
		break;

	case 8:
		parity_byte |= 0x03;
		break;
	}

switch (stop_bits)                         // bit 2 (2^2)
	{
	case 1:
		parity_byte |= 0x00;
		break;

	case 2:
		parity_byte |= 0x04;
		break;
	}

return parity_byte;
}

/*---------------------------------------------------------------------------*/
// execute the code at the passed funtion pointer
/*---------------------------------------------------------------------------*/
int put_char(int portnum, unsigned char ch)
{
return (*putCharTask)(portnum, ch);
}

/*---------------------------------------------------------------------------*/
// execute the code at the passed funtion pointer
/*---------------------------------------------------------------------------*/
int get_char(int portnum)
{
return (*getCharTask)(portnum);
}

/*---------------------------------------------------------------------------*/
// execute the code at the passed funtion pointer
/*---------------------------------------------------------------------------*/
int bytesWaiting(int portnum)
{
return (*inputQueueTask)(portnum);
}

/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
// Receive a character
/*---------------------------------------------------------------------------*/
int getArnetChar(int portnum) 
{
char buf[128];
union REGS inregs, outregs;
struct SREGS segregs;

/* set up registers for INT 14 */
inregs.h.ah = READ_BYTE;
inregs.x.dx = portnum;
inregs.x.si = 0;

int86x(RSINT, &inregs, &outregs, &segregs);

if (outregs.h.ah & 0x80)
	{
	sprintf(buf, "Timeout: port %d\n", portnum);
	logMsg(buf);
	scrollMsg(buf);
	return -1;
	}

return (int) outregs.h.al;
}




/*---------------------------------------------------------------------------*/
// Get a string from the specified port
/*---------------------------------------------------------------------------*/
int read_string(int portnum, char * string, int bytes)
{
union REGS inregs, outregs;
struct SREGS segregs;

inregs.h.ah = READ_STRING;
inregs.h.al = 0;					 // wait for buffer to be transmitted
inregs.x.cx = (unsigned int) bytes;
inregs.x.dx = (unsigned int) portnum;
inregs.x.bx = FP_OFF(string);
segregs.es = FP_SEG(string);
inregs.x.si = 0;

int86x(RSINT, &inregs, &outregs, &segregs);
return outregs.x.cx;
}




/*---------------------------------------------------------------------------*/
// Read a string from the specified port
// Returns:    -1 if timeout or no bytes can be read
// Returns:    -2 More than 255 bytes were read without a newline
// Returns:    -3 If a non-ascii character was encountered
// Returns:    -4 If an alpha character was not upper case
// Returns:     0 if Complete Line has been read
// Returns:    strlen() if bytes were added but not complete yet
/*---------------------------------------------------------------------------*/
int gets_port(t_com * p)
{
register int ch;
int retv = 0;
int i = 0;
char temp[MAX_PORT_BUF_SIZE];
static char buf[MAX_PORT_BUF_SIZE];
static int bufTail[MAX_COMPORTS];                // pointer to end of buffer
int bytes_to_check;
int hold_bytes;

hold_bytes = p->inputQueue = (*inputQueueTask)(p->portnum);

// Nothing to do ??
if (p->inputQueue == 0)
	{
	// printf("Queue empty p->inputQueue = %d port %d\n", p->inputQueue, p->portnum);
	return -1;
	}

// read the buffer in all at once
retv = read_string(p->portnum, &buf[bufTail[p->portnum]], p->inputQueue);
if (retv != (int) p->inputQueue)
	{
	printf("read_string failed %s at %d retv %d bytes %d\n", 
          __FILE__, __LINE__, retv, p->inputQueue);
	return -1;
	}

i = 0;
bytes_to_check = bufTail[p->portnum] + retv;     // read old & new bytes
// printf("bytes_to check at start of loop: %d\n", bytes_to_check);
do 
	{
	if (!(ch = buf[i++]) )
		{
		printf("get_char failed bytes %d i %d retv %d\n", hold_bytes, i, retv);
		bufTail[p->portnum] = 0;
		p->tail = 0;                          // get ready for next try
		return -1;
		}
	
	if (!isascii(ch))
		{
		p->tail = 0;                          // get ready for next try
		bufTail[p->portnum] = 0;
		return -3; 
		}
	
	// all characters must be upper case
	if (isalpha(ch))
		{
		if (!isupper(ch))
			{
			p->tail = 0;                          // get ready for next try
			bufTail[p->portnum] = 0;
			return -4; 
			}
		}

	p->buf[p->tail++] = (char) ch;         // add the char
	// printf("%c %02X    ", ch, ch);

	} while (                             // do this as long as ....
          ( p->tail < MAX_PORT_BUF_SIZE ) &&    // string is not too long
          ( i < bytes_to_check )    &&          // bytes remaining
          ( ch != LF )    &&          // we didn't get a newline
          ( ch != CR )    &&          // we didn't get a newline
          ( ch != '\n' )    &&          // we didn't get a newline
          ( ch != '\0' ) );           // we didn't get a NULL

// see why we dropped out
// printf("\nAfter loop p->tail = %d\n", p->tail);
// printf("bytes = %d\n", p->inputQueue);
// printf("ch = %02X\n", ch);
// printf("bytes_to_check = %d\n", bytes_to_check);
// printf("hold_bytes = %d\n", hold_bytes);
// printf("i = %d\n", i);

p->buf[p->tail] = '\0';                 // Always keep it Null-terminated

// Now check to see if we got all the chars out of buf
if (i < bytes_to_check)          // then we still have bytes in static buf
	{
	// Save them for the next read
	strset(temp, '\0');
	strncpy(temp, buf+i, bytes_to_check-i);  
	strset(buf, '\0');
	strcpy(buf, temp);
	bufTail[p->portnum] = strlen(buf);
	// printf("There are bytes remaining in BUF!!!\n");
	// printf("temp >>%s<<\n", temp);
	// printf("buf >>%s<<\n", buf);
	// printf("Setting bufTail %d\n", bufTail[p->portnum]);
	}
else                          // all the chars in buf were read
	{
	bufTail[p->portnum] = 0;
	}


if (p->tail >= 255)
	{
	p->tail = 0;                          // get ready for next try
	return -2;                            // tell caller something went wrong
	}

if (ch == '\n') 
	{
	p->tail = 0;                           // get ready for next read
	return SUCCESS;
	}

return (int) strlen(p->buf);
}






/*---------------------------------------------------------------------------*/
// See how many bytes are waiting in Serial port queue
/*---------------------------------------------------------------------------*/
unsigned int iq_check(int portnum)  
{
union REGS inregs, outregs;
struct SREGS segregs;

inregs.h.ah = IQCHK;	 
inregs.h.al = 0;

inregs.x.dx = FP_OFF(portnum);
segregs.ds = FP_SEG(portnum);

int86x(RSINT, &inregs, &outregs, &segregs);
return (int) outregs.x.ax;
}





/*---------------------------------------------------------------------------*/
// Disable All serial Ports
/*---------------------------------------------------------------------------*/
void disable_serial_ports(void)
{
int i;

for (i=0; i<MAX_COMPORTS; i++)
	{
	disable_one_port(i);
	}

if (!_SmartPortInstalled)
	{
	ecTermComm();
	}

}




/*---------------------------------------------------------------------------*/
// Set port handshaking
/*---------------------------------------------------------------------------*/
int set_handshake(int port, int handshake)
{
union REGS inregs, outregs;
struct SREGS segregs;

inregs.h.ah = SET_HANDSHAKE;
inregs.h.al = (unsigned char) handshake;
inregs.x.dx = (unsigned int) port;
inregs.h.bl = (unsigned char) XON;
inregs.h.bh = (unsigned char) XOFF;
int86x(RSINT, &inregs, &outregs, &segregs);

// send XON char
if (handshake == XON_XOFF)
	put_char(port, XON);

return outregs.h.ah;
}


/*---------------------------------------------------------------------------*/
// Disable One Serial Port
/*---------------------------------------------------------------------------*/
void disable_one_port(int portnum)
{
union REGS inregs, outregs;
struct SREGS segregs;

if (_SmartPortInstalled)
	{
	inregs.h.ah = DISABLE;
	inregs.x.dx = (unsigned int) portnum;
	int86x(RSINT, &inregs, &outregs, &segregs);
	}
else
	{
	ecDisablePort(portnum);
	}

}



/*---------------------------------------------------------------------------*/
// Initialize all enabled serial ports
/*---------------------------------------------------------------------------*/
void initAllPorts(void)
{
t_com * p;
int i;
char buf[128];

for (i=0; i<MAX_COMPORTS; i++)
	{
	p = &com_struct[i];
	if (p->enabled)
		{
		sprintf(buf, "Initializing %s for %s", p->portname, p->sensor_name);
		Notify(-1, -1, buf);

		if (p->sensor_index == TRIMBLE_TANS)
			initTans(p->portnum);

		set_baud(p->portnum, p->baud, p->parity, p->data, p->stop, p->handshake);
		NotifyOff();
		}
	}
}


/*----------------------------------------------------------------------------*/
// read all serial ports
/*----------------------------------------------------------------------------*/
void readAllPorts(void)
{
int i;
int bytes = 0;
int result;

for (i=0; i<MAX_COMPORTS; i++)
	{
	if (com_struct[i].enabled)
		{
		result = readPort(&com_struct[i]);
			
		if (result == SUCCESS)
			{
			parse(&com_struct[i]);
			recordNav(&com_struct[i]);
			}   // end of if result
		}   // end of if enabled
	}   // end of for loop
}



/*----------------------------------------------------------------------------*/
// send all serial ports
/*----------------------------------------------------------------------------*/
void sendAllPorts(void)
{
int i;
static double lastXmitTime[MAX_COMPORTS];
double now = systemTime();
char buf[512];
t_nav * pNav;

for (i=0; i<MAX_COMPORTS; i++)
	{
	if (com_struct[i].enabled)
		{
		if (now - lastXmitTime[i] > (double) com_struct[i].interval)
			{
			switch (com_struct[i].sensor_index)
				{
				case TRACKPOINT_TO_URI:
					// get the active trackpoint port
					pNav = queryNavByType(TRACKPOINT_II);
					if (pNav != (t_nav *) NULL)
						{
						// build the outgoing packet
						if (buildVaxRecord(pNav, buf) != (char *) NULL)
							{
							sendString(com_struct[i].portnum, buf);
							put_char(com_struct[i].portnum, CR);
							put_char(com_struct[i].portnum, LF);
							lastXmitTime[i] = now;
							}
						}
				break;

				case DDMU_TO_URI:
					// get the active Delnorte port
					pNav = queryNavByType(DDMU540);
					if (pNav != (t_nav *) NULL)
						{
						// build the outgoing packet
						if (buildVaxRecord(pNav, buf) != (char *) NULL)
							{
							sendString(com_struct[i].portnum, buf);
							put_char(com_struct[i].portnum, CR);
							put_char(com_struct[i].portnum, LF);
							lastXmitTime[i] = now;
							}
						}
					break;
	
				case BRIDGE_TERM:
				case MUDSHARK:
					buildBridgePacket(i);
					sendBridgePacket(i);
					lastXmitTime[i] = now;
					break;
	
				default:
					break;
				}   // end of switch
			}   // end of if enabled
		} // end of if now - ....
	}   // end of for loop
}



/*----------------------------------------------------------------------------*/
// Parse out the Trimble data
/*----------------------------------------------------------------------------*/
void parse_trimble(t_com * p)
{
char lat_str[30];
char lon_str[30];
char date_str[20];
char month_str[10];
unsigned hour, minute, second;
int day;
int year;
int alt;
time_t ltime;
char diffMode = FALSE;
int retv;
struct tm gpsTime;

if ( (p->buf[strlen(p->buf) - 3] != ']') &&
     (p->buf[strlen(p->buf) - 2] != ']')  )          // for simulator
	{
	return;
	}

// see if it is differential
if (p->buf[30] == 'd')
	{
	diffMode = TRUE;
	}
WhiteOutChar(p->buf, 'd');

// memset((void *) &navSensors[p->portnum], 0x00, sizeof(t_nav));

/**********        the Trimble data looks like this in autonomous mode
[00 TUE 294 22-OCT-91 11:01:22 37:52.9984N 122:38.9757W -0022 03.0 485233 -000.01 000.00 000.0 +1.4999E-08 020 4 13,11,08,09]
***********/

/**********        the Trimble data looks like this in differential mode
[00 TUE 294 22-OCT-91 11:01:22d37:52.9984N 122:38.9757W -0022 03.0 485233 -000.01 000.00 000.0 +1.4999E-08 020 4 13,11,08,09]
************/

retv = sscanf(p->buf, "%*s %*s %*d %s %u:%u:%u %s %s %d %lf %*ld %*f %lf %*lf %*lf %*d %*d %*s",
          date_str,
          &hour,
          &minute,
          &second,
          lat_str,
          lon_str,
          &alt,
          &navSensors[p->portnum].msmts.gps.pdop,
          &navSensors[p->portnum].spd);

if (retv != 9)
	{
	scrollMsg("Bad Scanf\n");
	return;
	}

// strip out the date string
WhiteOutChar(date_str, '-');
sscanf(date_str, "%d %s %d", &day, month_str, &year);
	
// build up the time structure
gpsTime.tm_year = year;
gpsTime.tm_mon = month_str_to_int(month_str);
gpsTime.tm_mday = day;
gpsTime.tm_hour = hour;
gpsTime.tm_min = minute;
gpsTime.tm_sec = second;
gpsTime.tm_isdst = FALSE;

// sync computer time to GPS
ltime = mktime(&gpsTime);
if ((ltime % 3600L) == 0)
	SyncTime(ltime);

PositionStrToRads(lat_str, &navSensors[p->portnum].lat, LAT);
PositionStrToRads(lon_str, &navSensors[p->portnum].lon, LON);
// navSensors[p->portnum].alt = (double) alt;
navSensors[p->portnum].tim = systemTime();
navSensors[p->portnum].id = TRIMBLE_4000;
navSensors[p->portnum].msmts.gps.dgps = diffMode;
}




/*----------------------------------------------------------------------------*/
// Parse out the Trimble TANS
/*----------------------------------------------------------------------------*/
void parseTans(t_com * p)
{
t_tans * pTans = queryTansStruct();

if (pTans == (t_tans *) NULL)
	return;

navSensors[p->portnum].msmts.gps.pdop = pTans->gps.pdop;
navSensors[p->portnum].lat = pTans->gps.pos.lla.lat;
navSensors[p->portnum].lon = pTans->gps.pos.lla.lon;
navSensors[p->portnum].tim = systemTime();
navSensors[p->portnum].id = TRIMBLE_TANS;
}


/*----------------------------------------------------------------------------*/
// Parse out the Level A Gyro data
// 1992 055 09:40:05              353.6                                
/**************************************************************************
      DRT               
                        
    LAT N  47 35.398    
    LON W 122 20.535    
 01 20 28      
                        
M SPEED  0.0 M HDG 000.0
                        
                MAGNAVOX
*****************************************************************************/
/*----------------------------------------------------------------------------*/
void parse_mx1107(t_com * p)
{
char buf[256];
double spd, hdg;

if (strstr(p->buf, "LAT"))
	{
	strcpy(buf, p->buf);
	replaceMultiChar(buf, "LAT", ' ');
	PositionStrToRads(buf, &navSensors[p->portnum].lat, LAT);
	}

if (strstr(p->buf, "LON"))
	{
	strcpy(buf, p->buf);
	replaceMultiChar(buf, "LON", ' ');
	PositionStrToRads(buf, &navSensors[p->portnum].lon, LON);
	}
	
if (strstr(p->buf, "HDG"))
	{
	strcpy(buf, p->buf);
	replaceMultiChar(buf, "SPEDMHDGA", ' ');
	sscanf(buf, "%lf %lf", &spd, &hdg);
	navSensors[p->portnum].hdg = hdg * DEG_TO_RAD;
	}

// load up the structure
navSensors[p->portnum].id  = (unsigned char) MAGNAVOX_1107;
navSensors[p->portnum].tim = systemTime();
navSensors[p->portnum].crs = navSensors[p->portnum].hdg;
navSensors[p->portnum].spd = spd * KTS_TO_MPS;
}


/*----------------------------------------------------------------------------*/
// Parse out the Trimble NavTrac GPS receiver NMEA output
// $GPGGA,213832,3730.4487,N,12212.6442,W,1,3,001,*67
/*----------------------------------------------------------------------------*/
void parse_navtrac(t_com * p)
{
char idString[60];
char timBuf[60];
char latBuf[60];
char latHem;
char lonHem;
char lonBuf[60];
char checkSum[52];
int GPS_available=0;
int satsUsed = 0;
int Hdop = 0;
int commas;
int fieldsRead;
int j=0;
char buf[512];
struct tm gpsTime;
unsigned hour, minute, second;
time_t now = time(NULL);

// $GPGGA,213832,3730.4487,N,12212.6442,W,1,3,001,*67
commas = WhiteOutChar(p->buf, ',');
fieldsRead = sscanf(p->buf, "%s %s %s %c %s %c %d %d %d %s",
            idString,
            timBuf,
            latBuf,
            &latHem,
            lonBuf,
            &lonHem,
            &GPS_available,
            &satsUsed,
            &Hdop,
            checkSum);

/*************
printf("1. latbuf >>%s<<    lonBuf >>%s<<\n", latBuf, lonBuf);
if ((fieldsRead-1 != commas) || (!GPS_available))
	{
	scrollMsg("NavTrac Not Tracking\n");
	return ;
	}
*************/
            
// build up the time structure using the current time
memcpy((void *) &gpsTime, (void *) gmtime(&now), sizeof(gpsTime));

// read in the hour minute second
sscanf(timBuf, "%2d%2d%2d", &hour, &minute, &second);

// replace those values in the structure
gpsTime.tm_hour = hour;
gpsTime.tm_min = minute;
gpsTime.tm_sec = second;
gpsTime.tm_isdst = FALSE;

// sync computer time to GPS
now = mktime(&gpsTime);
if ((now % 600L) == 0)
	SyncTime(now);

// add the Hemisphere to the end of the position
strcpy(latBuf, insertCharAtPos(latBuf, 40, (int) latHem, buf));

// separate the degrees from the minutes
PositionStrToRads( insertCharAtPos(latBuf, 2, ' ', buf), 
                   &navSensors[p->portnum].lat, 
                   LAT);

// add the Hemisphere to the end of the position
strcpy(lonBuf, insertCharAtPos(lonBuf, 40, (int) lonHem, buf));

// separate the degrees from the minutes
PositionStrToRads( insertCharAtPos(lonBuf, 3, ' ', buf), 
                   &navSensors[p->portnum].lon, 
                   LON);
// printf("2. latBuf >>%s<<    lonBuf >>%s<<\n", latBuf, lonBuf);

// load up the structure
navSensors[p->portnum].id  = (unsigned char) TRIMBLE_NAVTRAK;
navSensors[p->portnum].tim = systemTime();
navSensors[p->portnum].msmts.gps.pdop = Hdop;
navSensors[p->portnum].msmts.gps.sat_count = satsUsed;
}




/*----------------------------------------------------------------------------*/
// Parse out the KVH C100 Gyro NMEA 0183 data
// $HCHDM,003.6,M*2C
/*----------------------------------------------------------------------------*/
void parse_kvh_gyro(t_com * p)
{
double hdg;
char idString[80];

// $HCHDM,003.6,M*2C
WhiteOutChar(p->buf, ',');
sscanf(p->buf, "%s %lf",
            &idString,
            &hdg);

// load up the structure
navSensors[p->portnum].id  = (unsigned char) KVH_GYRO;
navSensors[p->portnum].tim = systemTime();
navSensors[p->portnum].hdg = hdg * DEG_TO_RAD;
navSensors[p->portnum].crs = navSensors[p->portnum].hdg;

// printf("%s: idString %s hdg %.1lf\n", p->buf, idString, hdg);
}



/*----------------------------------------------------------------------------*/
// Parse out the CEC Buster Gyro digitizer
// 15911 (degrees * 100.0)
/*----------------------------------------------------------------------------*/
void parse_buster_gyro(t_com * p)
{
// load up the structure
navSensors[p->portnum].hdg = (atof(p->buf) / 100.0) * DEG_TO_RAD;
navSensors[p->portnum].crs = navSensors[p->portnum].hdg;
navSensors[p->portnum].id  = (unsigned char) BUSTER_GYRO;
navSensors[p->portnum].tim = systemTime();

// printf("%s hdg %.2lf\n", p->buf, navSensors[p->portnum].hdg * RAD_TO_DEG);
}



/*----------------------------------------------------------------------------*/
// Parse out the Raytheon Bathy Digitizer PDD
// 00324                == 32.4 meters
/*----------------------------------------------------------------------------*/
void parse_pdd(t_com * p)
{
t_setup * config = QuerySetup();
long rawDepth;

// load up the structure
navSensors[p->portnum].lat = curNav->lat;
navSensors[p->portnum].lon = curNav->lon;
navSensors[p->portnum].hdg = (atof(p->buf) / 100.0) * DEG_TO_RAD;
navSensors[p->portnum].crs = navSensors[p->portnum].hdg;
navSensors[p->portnum].id  = (unsigned char) PDD_DEPTH_DIGITIZER;
navSensors[p->portnum].tim = systemTime();
sscanf(p->buf, "%ld", &rawDepth);

depthAlarmFlag = !readIOLine(PDD_ALARM_IO_LINE);

navSensors[p->portnum].msmts.pdd.rawDepth = (double) rawDepth / 10.0;
if (config->depth.depthCorrectionEnabled)
	{
	navSensors[p->portnum].msmts.pdd.corDepth = 
                                    navSensors[p->portnum].msmts.pdd.rawDepth /
                                    PDD_SOUND_VELOCITY * 
                                    config->depth.soundVelocity;
	curNav->alt = navSensors[p->portnum].alt = 
                navSensors[p->portnum].msmts.pdd.corDepth;
	}
else
	{
	navSensors[p->portnum].msmts.pdd.corDepth = 0.0;
	curNav->alt = navSensors[p->portnum].alt = 
                navSensors[p->portnum].msmts.pdd.rawDepth;
	}

navSensors[p->portnum].msmts.pdd.soundVel = config->depth.soundVelocity;
navSensors[p->portnum].msmts.pdd.alarm    = depthAlarmFlag;
}




/*----------------------------------------------------------------------------*/
// Parse the packet, aka the case statement from hell.....
/*----------------------------------------------------------------------------*/
int parse(t_com * p)
{
scrollMsg(p->sensor_name);
scrollMsg("\n");

switch(p->sensor_index)
	{
	case ASHTECH:
		parse_ashtech(p);
		break;

	case NS7000:
		parse_ns7000(p);
		break;

	case MAGNAVOX_1107:
		parse_mx1107(p);
		break;

	case TRACKPOINT_II:
		if (parse_trackpointII(p) == SUCCESS)
			{
			// sendFishData(p);
			}
		break;

	case TRIMBLE_4000:
		parse_trimble(p);
		break;

	case TRIMBLE_TANS:
		parseTans(p);
		break;

	case DDMU540:
		parse_ddmu540(p);
		break;

	case BUSTER_GYRO:
		parse_buster_gyro(p);
		break;

	case KVH_GYRO:
		parse_kvh_gyro(p);
		break;

	case TRIMBLE_NAVTRAK:
		parse_navtrac(p);
		break;

	case PDD_DEPTH_DIGITIZER:
		parse_pdd(p);
		break;
	}
return TRUE;
}



/*----------------------------------------------------------------------------*/
// Parse the Northstar 7000 datagram
/*----------------------------------------------------------------------------*/
int parse_ns7000(t_com * p)
{
int hour, min, sec;
time_t now = time(NULL);
time_t midnite = now - (now % 86400L);     // get the time at midnite
// int j;
// char buf[512];

// Sanity checks
if ('T' != p->buf[109])
	{
	// sprintf(buf, "Bad(%c%c%c%c%c) %s\n", p->buf[108],p->buf[109], p->buf[110], p->buf[111], p->buf[112], p->sensor_name);
	scrollMsg("Bad Ns7000\n");
	// memset((void *) &navSensors[p->portnum], 0x00, sizeof(t_nav));
	return -1;
	}

// memset((void *) &navSensors[p->portnum], 0x00, sizeof(t_nav));

// create a full dos time
hour = getInt(p->buf, 0, 2);
min  = getInt(p->buf, 2, 2);
sec  = getInt(p->buf, 4, 2);
midnite  += (long) hour * 3600L;
midnite  += (long) min * 60L;
midnite  += (long) sec;

navSensors[p->portnum].id = NS7000;
navSensors[p->portnum].msmts.lor.tds[0]  = getDbl(p->buf,  6, 8) / 1000.0;
navSensors[p->portnum].msmts.lor.tds[1]  = getDbl(p->buf, 19, 8) / 1000.0;
navSensors[p->portnum].msmts.lor.tds[2]  = getDbl(p->buf, 32, 8) / 1000.0;
navSensors[p->portnum].msmts.lor.tds[3]  = getDbl(p->buf, 45, 8) / 1000.0;
navSensors[p->portnum].lat = dms_to_rads(
                   getInt(p->buf, 58, 2),            // degrees
                   getDbl(p->buf, 60, 4) / 100.0,    // minutes
                   0.0,                              // seconds
                   p->buf[64]);                      // hemisphere

navSensors[p->portnum].lon = dms_to_rads(
                   getInt(p->buf, 67, 3),            // degrees
                   getDbl(p->buf, 70, 4) / 100.0,    // minutes
                   0.0,                              // seconds
                   p->buf[74]);                      // hemisphere
navSensors[p->portnum].msmts.lor.gri = getInt(p->buf, 77, 4);
navSensors[p->portnum].msmts.lor.masterToa = getDbl(p->buf, 81, 9) / 1000.0;
navSensors[p->portnum].spd =  getDbl(p->buf, 102, 4) / 10.0;
// navSensors[p->portnum].crs =  getDbl(p->buf, 106, 3) * DEG_TO_RAD;
// navSensors[p->portnum].crs *= DEG_TO_RAD;
navSensors[p->portnum].tim = systemTime();
// navSensors[p->portnum].tim =  midnite;

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Parse the Ashtech packet
/*----------------------------------------------------------------------------*/
int parse_ashtech(t_com * p)
{
char buf[256];         // scratch area

// $PASHR,PBN,145278.00,-2700294.0,-4286243.0,3862293.1,037:30.4748,-122:12.6332,0053.1,000.03,000.03,-000.09,04,????,20,10,17,15

// see if it is the right record
if (strncmp(p->buf, "$PASHR,PBN,", 10))
	{
	scrollMsg(p->buf);
	return FALSE;
	}

// memset((void *) &navSensors[p->portnum], 0x00, sizeof(t_nav));

// get the time
strset(buf, '\0');
strncpy(buf, p->buf+11, 9);
// navSensors[p->portnum].tim = GPStoDOS(atof(buf), 0);
navSensors[p->portnum].tim = systemTime();

// get the latdeg
strset(buf, '\0');
strncpy(buf, p->buf+53, 11);
PositionStrToRads(buf, &navSensors[p->portnum].lat, LAT);

// get the latdeg
strset(buf, '\0');
strncpy(buf, p->buf+65, 12);
PositionStrToRads(buf, &navSensors[p->portnum].lon, LON);

// get the geoidal height
// strset(buf, '\0');
// strncpy(buf, p->buf+78, 6);
// navSensors[p->portnum].alt = atof(buf);

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// Return a first com port index that is enabled and has passed nav sensor type
/*----------------------------------------------------------------------------*/
int queryComIdxByNav(int navType)
{
int i;

for (i=0; i<MAX_COMPORTS; i++)
	{
	if (com_struct[i].enabled)
		{
		if (com_struct[i].sensor_index == navType)
			return i;
		}
	}
return -1;
}



/*----------------------------------------------------------------------------*/
// Return a first com port that is enabled and has the passed nav sensor type
/*----------------------------------------------------------------------------*/
t_com * queryComByNav(int navType)
{
int i;

for (i=0; i<MAX_COMPORTS; i++)
	{
	if (com_struct[i].enabled)
		{
		if (com_struct[i].sensor_index == navType)
			return &com_struct[i];
		}
	}
return (t_com *) NULL;
}



/*----------------------------------------------------------------------------*/
// Return a the nav pointer associated with the nav Type
/*----------------------------------------------------------------------------*/
t_nav * queryNavByType(int sensorType)
{
t_com * p = queryComByNav(sensorType);

// See if a port is open and enabled for this sensor
if (p == (t_com *) NULL)
	{
	return (t_nav *) NULL;
	}

return (t_nav *) &navSensors[p->portnum];
}


/*----------------------------------------------------------------------------*/
// Return a the nav pointer associated with the passed com port
/*----------------------------------------------------------------------------*/
t_nav * queryNavPtr(int which)
{
if (which < 0 || which >= MAX_COMPORTS)
	return (t_nav *) NULL;

return (t_nav *) &navSensors[which];
}


/*----------------------------------------------------------------------------*/
// Return a pointer to the requested nav structure
/*----------------------------------------------------------------------------*/
int readPort(t_com * p )
{
int result = -1;

p->inputQueue = (*inputQueueTask)(p->portnum);

switch(p->sensor_index)
	{
	case KVH_GYRO:
		if (p->inputQueue >= KVH_GYRO_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case TRIMBLE_TANS:
		result = updateTans();
		break;

	case TRACKPOINT_II:
		if (p->inputQueue >= TRACKPOINT_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case NS7000:
		if (p->inputQueue >= NS7000_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case TRIMBLE_4000:
		if (p->inputQueue >= TRIMBLE_4000_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case DDMU540:
		if (p->inputQueue >= DDMU540_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case BUSTER_GYRO:
		if (p->inputQueue >= BUSTER_GYRO_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case MAGNAVOX_1107:
		if (p->inputQueue >= MAGNAVOX_1107_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case TRIMBLE_NAVTRAK:
		if (p->inputQueue >= NAVTRAK_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	case PDD_DEPTH_DIGITIZER:
		if (p->inputQueue >= PDD_DEPTH_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;

	default:
		if (p->inputQueue >= DEFAULT_REC_SIZE)
			{
			result = getNbytes(p);
			}
		break;
	}
return result;
}



/*---------------------------------------------------------------------------*/
// Read a string from the specified port a character at a time
// Returns:    -1 if timeout or no bytes can be read
// Returns:    -2 More than 255 bytes were read without a newline
// Returns:    -3 If a non-ascii character was encountered
// Returns:    -4 If an alpha character was not upper case
// Returns:     0 if Complete Line has been read
// Returns:    strlen() if bytes were added but not complete yet
/*---------------------------------------------------------------------------*/
int getNbytes(t_com * p)
{
register int ch;

// Nothing to do ??
if (p->inputQueue == 0)
	{
	// printf("Queue empty bytes = %d port %d\n", p->inputQueue, p->portnum);
	return -1;
	}

do {
	if ( (ch = get_char ( p->portnum )) == -1) 
		{
		printf("get_char failed\n");
		return ch;
		}

	if (!isascii(ch))
		{
		p->tail = 0;                          // get ready for next try
		return -3; 
		}

	p->buf[p->tail++] = (char) ch;         // add the char

	} while (                             // do this as long as ....
            ( p->tail < MAX_PORT_BUF_SIZE ) &&    // string is not too long
						(p->inputQueue--)  &&        // there are still bytes to be read
            ( ch != '\n' )    &&        // we didn't get a newline
            ( ch != '\0' ) );           // we didn't get a NULL

p->buf[p->tail] = '\0';                 // Always keep it Null-terminated

if (p->tail >= 255)
	{
	p->tail = 0;                          // get ready for next try
	return -2;                            // tell caller something went wrong
	}

if (ch == '\n') 
	{
	p->tail = 0;                           // get ready for next read
	return SUCCESS;
	}

return (int) strlen(p->buf);
}




/*----------------------------------------------------------------------------*/
// Parse out the Del Norte 540 message
/*----------------------------------------------------------------------------*/
int parse_ddmu540(t_com * p)
{
t_station * pStation ;
int trisponderIDs[MAX_TRISPO_RANGES];
double ranges[MAX_TRISPO_RANGES];
double lat = curNav->lat;
double lon = curNav->lon;
int retv;
int event;
int i;
int numObs = 0;
char buf[128];
t_offset * off = queryOffset(DDMU540);
double height;

// 686 900  4815.64 801  3615.80 802 15572.53 803 15251.04         
retv = sscanf(p->buf,"%3d %3d %lf %3d %lf %3d %lf %3d %lf",
       &event,
       &trisponderIDs[0],
       &ranges[0],
       &trisponderIDs[1],
       &ranges[1],
       &trisponderIDs[2],
       &ranges[2],
       &trisponderIDs[3],
       &ranges[3]);

// see if it parsed correctly
if (retv != 9)
	{
	sprintf(buf, "Bad Trispo msg (%d)\n", retv);
	scrollMsg(buf);
	return BAD_DDMU540;
	}

// logMsg(p->buf);

// clear out the old measurements
memset((void *) &navSensors[p->portnum].msmts, 0x00, sizeof(union msmts));

// build the observation matrix
for (i=0; i<MAX_TRISPO_RANGES; i++)
	{
	// See if the station exists
	pStation = queryStationByID(trisponderIDs[i]);
	if (pStation == (t_station *) NULL)
		{
		sprintf(buf, "Received range for Unknown Transponder ID %03d\n", 
            trisponderIDs[i]);
		scrollMsg(buf);
		}
	else
		{
		// Build the observation structure
		if ( (pStation->enabled) && (ranges[i] != 0.0))
			{
			// adjust ranges for elevation
			height = off->z - pStation->height;
			ranges[i] = sqrt((ranges[i] * ranges[i]) - (height * height));
			ranges[i] += pStation->bias;          // Add the bias 
			navSensors[p->portnum].msmts.rng.obs.ranges[numObs] = ranges[i];
			navSensors[p->portnum].msmts.rng.obs.stations[numObs] = pStation;
			numObs++;
			navSensors[p->portnum].msmts.rng.obs.numObs = numObs;
			}
		}
	}

navSensors[p->portnum].tim = systemTime();
navSensors[p->portnum].id = DDMU540;

// clear the old nav values
// navSensors[p->portnum].lat = 0.0;
// navSensors[p->portnum].lon = 0.0;

// Are we calibrating ??
if (CalibrateOn)
	{
	computeCalibration( &navSensors[p->portnum].msmts.rng.obs, 
                     &Calib,
                     curNav);
	updateCalibrationFm(calibFm, 
                      &navSensors[p->portnum].msmts.rng.obs, 
                      curNav, 
                      &Calib);
	}

// don't compute if there is not enough info
if (numObs < 2)
	{
	scrollMsg("Too few Ranges\n");
	return NOT_ENOUGH_RANGES;
	}

// call the reduction function with the observations
if ((retv = reduceObservations( &navSensors[p->portnum].msmts.rng.obs,
                               &lat, &lon)) != SUCCESS)
	{
	sprintf(buf, "reduceObservations Failed (%d)\n", retv);
	scrollMsg(buf);
	return retv;
	}

navSensors[p->portnum].lat = lat;
navSensors[p->portnum].lon = lon;

return validNav(&navSensors[p->portnum]);
}


/*----------------------------------------------------------------------------*/
// Parse Trackpoint II
/*----------------------------------------------------------------------------*/
int parse_trackpointII(t_com * p)
{
char timeString[30];
char buf[512];
int retv;
double x = 0.0;
t_pos hydrophone;
double range;
double bearing;
double bazm;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid 

if (strstr(p->buf, "-.-") != (char *) NULL)
	{
	scrollMsg("TrkptII Data Incomplete\n");
	return BAD_TRACKPOINT;
	}

// 01234567890123456789012345678901234567890123456789012345678901234567890
// 0 13:54:30       2.5 86939.4   3819.4  86935.8  -790.4      0.0 56
retv = sscanf(p->buf, "%d %s %lf %lf %lf %lf %lf %lf",
        &navSensors[p->portnum].msmts.usbl.transponder_id,  // target #
        timeString,                                         // time string
        &navSensors[p->portnum].msmts.usbl.horiz_angle,     // rel bearing deg
        &navSensors[p->portnum].msmts.usbl.slant_range,     // slant range (m)
        &navSensors[p->portnum].msmts.usbl.x,               // relative x
        &navSensors[p->portnum].msmts.usbl.y,               // relative y
        &navSensors[p->portnum].msmts.usbl.z,               // relative z
        &navSensors[p->portnum].msmts.usbl.telemetry);      // depth


// see if it parsed correctly
if (retv != 8)
	{
	sprintf(buf, "Bad TrkptII data (%d)\n", retv);
	scrollMsg(buf);
	return BAD_TRACKPOINT;
	}

// pick up the error code if it's there
if (strlen(p->buf) >= 65)
	navSensors[p->portnum].msmts.usbl.err_codes = getInt(p->buf, 63, 2);

navSensors[p->portnum].id = TRACKPOINT_II;
navSensors[p->portnum].tim = systemTime();
navSensors[p->portnum].crs = curNav->crs;
navSensors[p->portnum].spd = curNav->spd;
navSensors[p->portnum].hdg = curNav->hdg;

// compute the position of the hydrophone from the VRP
computeOffset(TRACKPOINT_II, 
              &hydrophone.lat, 
              &hydrophone.lon);

// compute the horiz distance and bearing from Hydrophone to the fish
range = hypot(navSensors[p->portnum].msmts.usbl.x,             // relative x,y
              navSensors[p->portnum].msmts.usbl.y);
bearing = fmod(navSensors[p->portnum].msmts.usbl.horiz_angle * DEG_TO_RAD +
          curNav->hdg, TWOPI);

// YOOOOOOO delete this
// logMsg(p->buf);

// compute the actual fish location
retv = proj(hydrophone.lat, hydrophone.lon, 
            range, bearing, 
            &navSensors[p->portnum].lat,                // store fish position
            &navSensors[p->portnum].lon,                // store fish position
            ellipse->a, ellipse->b);
// sprintf(buf, "reported R&B from Trackpoint %.1lf %.1lf\n", range, bearing);
// logMsg(buf);

arclen(curNav->lat, curNav->lon,
       hydrophone.lat, hydrophone.lon,
       &range,
       &bearing,
       &bazm,
       ellipse->a,
       ellipse->b);
// sprintf(buf, "R&B from curPos to hydrophone %.1lf %.1lf\n", range, bearing);
// logMsg(buf);

arclen( hydrophone.lat, hydrophone.lon,
       navSensors[p->portnum].lat, 
       navSensors[p->portnum].lon, 
       &range,
       &bearing,
       &bazm,
       ellipse->a,
       ellipse->b);
// sprintf(buf, "R&B from hydrophone to fish %.1lf %.1lf\n", range, bearing);
// logMsg(buf);

return SUCCESS;
}


/*---------------------------------------------------------------------------*/
// Update the nav form with the bytes waiting information
/*---------------------------------------------------------------------------*/
void updateNavQueues(void)
{
int i;
int j=0;
char buf[128];
ftag * tag;

if (DisplayingSomething)
	return;

for (i=0; i<MAX_COMPORTS && j < 4; i++)
	{
	if (com_struct[i].enabled)
		{
		tag = (ftag *) navDispFm->ob_lst[11+j].tag;
		strcpy(tag->fld, itoa(com_struct[i].inputQueue, buf, 10));
		DrawTaggedOb(&navDispFm->ob_lst[11+j]);
		j++;
		}
	}
}


/*----------------------------------------------------------------------------*/
// dump the trackpoint data to a buffer
/*----------------------------------------------------------------------------*/
char * dumpTrackpoint(t_nav * p, char * buf)
{
int j = 0;

j += sprintf(buf+j, "transponder_id %d\n", p->msmts.usbl.transponder_id);
j += sprintf(buf+j, "x %.1lf  y %.1lf  z %.1lf\n", p->msmts.usbl.x,
                                                   p->msmts.usbl.y,
                                                   p->msmts.usbl.z);

j += sprintf(buf+j, "horiz_angle %.1lf\n", p->msmts.usbl.horiz_angle);
j += sprintf(buf+j, "slant_range %.1lf\n", p->msmts.usbl.slant_range);
j += sprintf(buf+j, "telemetry %.1lf\n", p->msmts.usbl.telemetry);
j += sprintf(buf+j, "error_codes %d\n", p->msmts.usbl.err_codes);

return buf;

}





/*---------------------------------------------------------------------------*/
// Transmit a character
/*---------------------------------------------------------------------------*/
int putArnetChar(int portnum, unsigned char ch) 
{
union REGS inregs, outregs;
struct SREGS segregs;

/* set up registers for INT 14 */
inregs.h.ah = XMIT_BYTE;
inregs.h.al = ch;
inregs.x.dx = portnum;
inregs.x.si = 0;

int86x(RSINT, &inregs, &outregs, &segregs);

if (outregs.h.ah & 0x80)
	{
	printf("Timeout while transmitting a byte\n");
	return SERIAL_IO_TIMEOUT;
	}

return SUCCESS;
}



/*---------------------------------------------------------------------------*/
// Send a string to the specified port
/*---------------------------------------------------------------------------*/
int sendString(int portnum, char * buf)
{
int i = 0;
unsigned char ch;

while (ch = *buf++)
	{
	put_char(portnum, ch);
	i++;
	}

return i;
}



/*---------------------------------------------------------------------------*/
// Send a buffer to the specified port using a TANS like protocol
/*---------------------------------------------------------------------------*/
int sendPacket(int portnum, int pkt_id, unsigned char buffer[], int size)
{
int i = 0;

if (size <= 0)
	return -1;

put_char(portnum, DLE);
put_char(portnum, (unsigned char) pkt_id);

while (size--)
	{
	put_char(portnum, buffer[i++]);

	// Send data DLE twice 
	if (buffer[i] == DLE)
		put_char(portnum, buffer[i]);
	}

put_char(portnum, DLE);
put_char(portnum, ETX);

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// send the nav packet to the bridge display
/*----------------------------------------------------------------------------*/
int sendBridgePacket(int idx)
{
int retv;

retv = sendPacket(idx,                                      // portnum
                  BRIDGE_TERM,                              // packet ID
                  (unsigned char *) &navSensors[idx],       // data 
                  sizeof(t_nav));                           // size of data

if (retv == sizeof(t_nav))
	return SUCCESS;
else
	return retv;
}




/*----------------------------------------------------------------------------*/
// build the nav packet for the bridge display
/*----------------------------------------------------------------------------*/
int buildBridgePacket(int idx)
{
// navSensors[idx].id =  BRIDGE_TERM;
navSensors[idx].id =  curNav->id;
navSensors[idx].tim = systemTime();
navSensors[idx].lat = curNav->lat;
navSensors[idx].lon = curNav->lon;
navSensors[idx].crs = curNav->crs;
navSensors[idx].spd = curNav->spd;
navSensors[idx].hdg = curNav->hdg;
memcpy(&navSensors[idx].msmts.navPacket.line, 
       queryTrackInfo(), 
       sizeof(t_track));
memcpy(&navSensors[idx].msmts.navPacket.site, 
       querySiteInfo(), 
       sizeof(t_rngBrg));

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// send the fish data to the appropriate recipient
/*----------------------------------------------------------------------------*/
void sendFishData(t_com * pCom)
{
t_com * dataSonicsCom = (t_com *) NULL;
t_com * URI_fishCom   = (t_com *) NULL;
t_nav * pNav = &navSensors[pCom->portnum];
char buf[128];
char buf2[128];
static time_t lastTime = 0L;
time_t now = time(NULL);
static char vaxBuf[256];

switch (pNav->msmts.usbl.transponder_id)
	{
	case 1:                         // Send this one to URI
		// build vax packet
		if (buildVaxRecord(pNav, vaxBuf) == (char *) NULL)
			{
			sprintf(buf, "%s:%d: %s\n", __FILE__, __LINE__, NULL_NAV_MSG);
			scrollMsg(buf);
			logMsg(buf);
			}
		else
			{
			sendString(pCom->portnum, vaxBuf);
			}

	case 2:                         // Send this one to Datasonics
		if (now - lastTime > FIVE_MINUTES)
			{
			lastTime = now;
			// find the data sonics port
			dataSonicsCom = queryComByNav(DATASONICS_CHIRP);
			if (dataSonicsCom == (t_com *) NULL)
				{
				scrollMsg("DataSonics Com Not Found\n");
				break;
				}
			else
				{
				sprintf(buf, "$LCGLL,%s,%s\n", nmeaLat(pNav->lat), nmeaLon(pNav->lon));
				sendString(dataSonicsCom->portnum, buf);
				sprintf(buf2, "E Fish Depth: %.1lf\n", pNav->msmts.usbl.telemetry);
				sendString(dataSonicsCom->portnum, buf2);
				}
			}
		break;
	}
}



/*---------------------------------------------------------------------------*/
// open and process the range calibartion form
/*---------------------------------------------------------------------------*/
int doCalibrateFm(ob_desc * ob)
{
int retv;
char buf[128];
t_setup * config = QuerySetup();
t_ellips * elps = query_ellipsoid();
char * frf = "calib.frf";

if ( ( calibFm = loadFRF ( frf ) ) == (fm_desc *) NULL ) 
	return FALSE;

// Fix bug in idt
calibFm->ob_lst[6].flags  |= IS_EDITABLE;
calibFm->ob_lst[7].flags  |= IS_EDITABLE;
calibFm->ob_lst[11].flags |= IS_EDITABLE;
calibFm->ob_lst[12].flags |= IS_EDITABLE;
calibFm->ob_lst[16].flags |= IS_EDITABLE;
calibFm->ob_lst[17].flags |= IS_EDITABLE;
calibFm->ob_lst[21].flags |= IS_EDITABLE;
calibFm->ob_lst[22].flags |= IS_EDITABLE;

if (!OpenForm( calibFm, 10, ob->area.Ymax))
	{
	sprintf(buf, "Failed to open form in %s at line %d\n", __FILE__, __LINE__);
	Puke(buf);
	}

// update the form with current calibration values

do 
	{
	retv = ProcessObList ( calibFm->ob_lst, calibFm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case CALIBRATE : 
			// set the flag to cause calibartion to occur
			CalibrateOn = TRUE;

			// Clear old station calibration values
			clearCalibrationValues();
			break;

		case DONE : 
			CalibrateOn = FALSE;
			break;

		case ACCEPT : 
			// update the station biases with the new values
			CalibrateOn = FALSE;
			updateCalibrationValues();
			break;
		}
	} while (retv != DONE && retv != ACCEPT);


CloseForm ( calibFm );
DestroyFRFForm(calibFm);
return  (retv == OKAY);
}



/*----------------------------------------------------------------------------*/
// Open if necessary and update the range calibration form
/*----------------------------------------------------------------------------*/
int updateCalibrationFm(fm_desc * fm, 
                        t_observations * pObs, 
                        t_nav * pNav, 
                        t_calibration * pCal)
{
int i, k;
ftag * namTag;
ftag * idTag;
ftag * truRngTag;
ftag * obsRngTag;
ftag * biasTag;
char buf[80];
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps

if (pNav == (t_nav *) NULL)
	return NULL_NAV;

if (pObs == (t_observations *) NULL)
	return NULL_OBSERVATIONS;

// Clear out the old values
for (i=0; i < (fm->ob_cnt - 1) / 5; i++)
	{
	namTag   = (ftag *) fm->ob_lst[5*i+3].tag;
	strset(namTag->fld, '\0');

	idTag    = (ftag *) fm->ob_lst[5*i+4].tag;
	strset(idTag->fld, '\0');

	truRngTag   = (ftag *) fm->ob_lst[5*i+5].tag;
	strset(truRngTag->fld, '\0');

	obsRngTag = (ftag *) fm->ob_lst[5*i+6].tag;
	strset(obsRngTag->fld, '\0');

	biasTag   = (ftag *) fm->ob_lst[5*i+7].tag;
	strset(biasTag->fld, '\0');

	// redraw the fields
	for (k=3; k<8; k++)
		{
		DrawTaggedOb(&fm->ob_lst[5 * i + k]);
		}
	}

// cycle through the elements in the form
for (i=0; i<pObs->numObs; i++)
	{
	namTag   = (ftag *) fm->ob_lst[5*i+3].tag;
	strcpy(namTag->fld, pObs->stations[i]->name);

	idTag    = (ftag *) fm->ob_lst[5*i+4].tag;
	strcpy(idTag->fld, itoa(pObs->stations[i]->id, buf, 10));

	truRngTag   = (ftag *) fm->ob_lst[5*i+5].tag;
	sprintf(obsRngTag->fld, "%.1lf", pObs->ranges[i]);

	obsRngTag = (ftag *) fm->ob_lst[5*i+6].tag;
	sprintf(truRngTag->fld, "%.1lf", pCal->ranges[i]);

	biasTag   = (ftag *) fm->ob_lst[5*i+7].tag;
	sprintf(biasTag->fld, "%.1lf", pCal->biases[i]);

	// redraw the fields
	for (k=3; k<8; k++)
		{
		DrawTaggedOb(&fm->ob_lst[5 * i + k]);
		}
	}

return SUCCESS;
}


/*----------------------------------------------------------------------------*/
// compute the calibration array
/*----------------------------------------------------------------------------*/
int computeCalibration(t_observations * pObs, 
                       t_calibration * pCal, 
                       t_nav * pNav)
{
double fazm, bazm;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid 
int i;

for (i=0; i< pObs->numObs; i++)
	{
	// compute the distance from passed nav to each station
	arclen(pNav->lat, 
         pNav->lon, 
         pObs->stations[i]->pos.lat, 
         pObs->stations[i]->pos.lon,
         &pCal->ranges[i], &fazm, &bazm, ellipse->a, ellipse->b);

	// compute the biases, subtract out the current station bias
	pCal->biases[i] = pCal->ranges[i] - 
                   (pObs->ranges[i] + pObs->stations[i]->bias);
	pCal->ids[i] = pObs->stations[i]->id;
	}

return pObs->numObs;
}



/*----------------------------------------------------------------------------*/
// put the computed calibration values in the station structures
/*----------------------------------------------------------------------------*/
void clearCalibrationValues(void)
{
t_station * pStation = (t_station *) NULL;
int i;

for (i=0; i<MAX_NAV_STATIONS; i++)
	{
	pStation = queryStationByID(Calib.ids[i]);
	if (pStation != (t_station *) NULL)
		{
		if (pStation->enabled)
			{
			pStation->bias = 0.0;
			}
		}
	}

return;
}


/*----------------------------------------------------------------------------*/
// put the computed calibration values in the station structures
/*----------------------------------------------------------------------------*/
void updateCalibrationValues(void)
{
t_station * pStation = (t_station *) NULL;
int i;

for (i=0; i<MAX_NAV_STATIONS; i++)
	{
	pStation = queryStationByID(Calib.ids[i]);
	if (pStation != (t_station *) NULL)
		{
		if (pStation->enabled)
			{
			pStation->bias = Calib.biases[i];
			}
		}
	}

return;
}




/*----------------------------------------------------------------------------*/
// set the read and write tasks
/*----------------------------------------------------------------------------*/
void setIOTasks(int (*readTask)(int), 
                int (*writeTask)(int, unsigned char),
                unsigned int (*queueTask)(int))
{
getCharTask      = readTask;
putCharTask      = writeTask;
inputQueueTask   = queueTask;
}
