/*----------------------------------------------------------------------------*/
// File: stations.c   USGS Navigation, allow user to manipulate stations
// Author: John Gann
// Start Date: 13 March 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "utmlib.h"
#include "nav.h"                 // Prototypes
#include "lines.h"              // Prototypes
#include "nav_util.h"           // Prototypes
#include "mnutils.h"           // Prototypes
#include "stations.h"           // Prototypes

// form constants
#define PREVIOUS 1
#define NEXT     2
#define SAVE     4

static t_station navStations[MAX_NAV_STATIONS];
static fm_desc * stationFm;
static char StationsAreLoaded = FALSE;

// externals
extern char DisplayingSomething;
extern char RefreshRequired;
//extern int sp_to_ll(double y, double x, double * y_rads, double * x_rads);


/*----------------------------------------------------------------------------*/
// print a station
/*----------------------------------------------------------------------------*/
void printStation(t_station * aStation)
{
char x_buf[40];
char y_buf[40];
char z_buf[40];

PositionString(x_buf, y_buf, z_buf, aStation->pos.lon, aStation->pos.lat, 0.0, 0);
fprintf(stdprn, "%-25s (%s) %-18s %-18s %9.1lf %03d\n", 
                aStation->name, 
                (aStation->enabled) ? "enabled" : "disabled",
                y_buf, x_buf, 
                aStation->height, aStation->id);
}



/*---------------------------------------------------------------------------*/
// The station Form
/*---------------------------------------------------------------------------*/
int doStationForm(ob_desc * ob)
{
static int idx=0;
int retv;
char buf[128];
t_setup * config = QuerySetup();
t_ellips * elps = query_ellipsoid();
char title[80];

if (!StationsAreLoaded)
	{
	if (!read_stations())
		{
		}
	}

if ( ( stationFm = loadFRF ( "station.frf" ) ) == (fm_desc *) NULL ) 
	return FALSE;

sprintf(title, "Station %d of %d", idx+1, MAX_NAV_STATIONS);
stationFm->title = (char *) title;

// Fix bug in idt
stationFm->ob_lst[8].flags  |= IS_EDITABLE;
stationFm->ob_lst[9].flags  |= IS_EDITABLE;
stationFm->ob_lst[11].flags |= IS_EDITABLE;

if (!OpenForm( stationFm, 10, ob->area.Ymax))
	{
	sprintf(buf, "Failed to open form in %s at line %d\n", __FILE__, __LINE__);
	Puke(buf);
	}

// Initialize the form with the passed line
UpdateStationPosition(stationFm, &navStations[idx]);

SetFldExitTask( PositionExitTask );

do 
	{
	retv = ProcessObList ( stationFm->ob_lst, stationFm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY : 
			if (getStationFields(idx) != SUCCESS)
				{
				Beep();
				retv = 99;         // stay in loop
				}
			break;

		case SAVE : 
			if (getStationFields(idx) != SUCCESS)
				{
				Beep();
				retv = 99;         // stay in loop
				}
			else           // advance to next station
				{
				idx++;
				if (idx >= MAX_NAV_STATIONS)
					idx = 0;
				UpdateStationPosition(stationFm, &navStations[idx]);
				sprintf(title, "Station %d of %d", idx+1, MAX_NAV_STATIONS);
				stationFm->title = (char *) title;
				DrawForm(stationFm);
				}
			break;

		case PREVIOUS : 
			idx--;
			if (idx < 0)
				idx = MAX_NAV_STATIONS - 1;
			UpdateStationPosition(stationFm, &navStations[idx]);
			sprintf(title, "Station %d of %d", idx+1, MAX_NAV_STATIONS);
			stationFm->title = (char *) title;
			DrawForm(stationFm);
			break;

		case NEXT : 
			idx++;
			if (idx >= MAX_NAV_STATIONS)
				idx = 0;
			UpdateStationPosition(stationFm, &navStations[idx]);
			sprintf(title, "Station %d of %d", idx+1, MAX_NAV_STATIONS);
			stationFm->title = (char *) title;
			DrawForm(stationFm);
			break;

		case CANCEL + 1: 
			break;
		}
	} while (retv != OKAY && retv != CANCEL + 1);


CloseForm ( stationFm );
DestroyFRFForm(stationFm);
RefreshRequired = (char) (retv == OKAY);
return  (retv == OKAY);
}





/*---------------------------------------------------------------------------*/
// Update the station array with the passed values if they are 
/*---------------------------------------------------------------------------*/
int getStationFields(int idx)
{
char x_buf[30];         /* String for Lon or Easting */
char y_buf[30];         /* String for Lat or Northing */
char z_buf[30];
ftag * tag;
int zone_num;
char zone_ltr;

zone_num = grid_zone_EW(navStations[idx].pos.lon);
zone_ltr = grid_zone_NS(navStations[idx].pos.lat);

/* See if all of the fields are valid */
tag = (ftag *) stationFm->ob_lst[6].tag;          // lat
strcpy(y_buf, tag->fld);
tag = (ftag *) stationFm->ob_lst[7].tag;          // lon
strcpy(x_buf, tag->fld);
tag = (ftag *) stationFm->ob_lst[8].tag;          // alt
strcpy(z_buf, tag->fld);

if (PositionXYToRads(y_buf, x_buf, 
                     &navStations[idx].pos.lat, 
                     &navStations[idx].pos.lon, 
                     zone_num, zone_ltr))
	{
	return BAD_POSITION;        /* Stay in LOOP */
	}

// Get the station name
tag = (ftag *) stationFm->ob_lst[5].tag;          // name
strcpy(navStations[idx].name, tag->fld);

tag = (ftag *) stationFm->ob_lst[8].tag;         // height
navStations[idx].height = atof(tag->fld);

tag = (ftag *) stationFm->ob_lst[11].tag;        // bias
navStations[idx].bias = atof(tag->fld);

tag = (ftag *) stationFm->ob_lst[9].tag;           // id
navStations[idx].id = atoi(tag->fld);

if (navStations[idx].id >= 100)
	{
	navStations[idx].id = navStations[idx].id / 10;
	}

if (stationFm->ob_lst[10].flags & IS_CHECKED)       // enabled
	navStations[idx].enabled = TRUE;
else
	navStations[idx].enabled = FALSE;

return SUCCESS;
}



/*---------------------------------------------------------------------------*/
/* Update the coordinate fields on the Position entry form                    */
// Offset: 0=starting coordinates, 1=ending coordinates
/*---------------------------------------------------------------------------*/
void UpdateStationPosition(fm_desc * fm, t_station * station)
{
char x_buf[30];         /* String for Lon or Easting */
char y_buf[30];         /* String for Lat or Northing */
char z_buf[30];
ftag * tag;

PositionString(x_buf, y_buf, z_buf, 
              station->pos.lon, 
              station->pos.lat, 
              station->height, 
              0);
			
tag = (ftag *) fm->ob_lst[6].tag;          /* Copy Y Value to form */
strcpy(tag->fld, y_buf);
	
tag = (ftag *) fm->ob_lst[7].tag;          /* Copy X Value to form */
strcpy(tag->fld, x_buf);
	
tag = (ftag *) fm->ob_lst[8].tag;          /* Copy Z Value to form */
strcpy(tag->fld, z_buf);

tag = (ftag *) fm->ob_lst[5].tag;          // the name
strcpy(tag->fld, station->name);

tag = (ftag *) fm->ob_lst[9].tag;          // id
sprintf(tag->fld, "%d", station->id * 10 + 4);

tag = (ftag *) fm->ob_lst[11].tag;          // id
sprintf(tag->fld, "%.1lf", station->bias);

if (station->enabled)
	{
	fm->ob_lst[10].flags |= IS_CHECKED;
	}
else
	{
	fm->ob_lst[10].flags &= ~IS_CHECKED;
	}
	
DrawTaggedOb(&fm->ob_lst[5]);      // name 
DrawTaggedOb(&fm->ob_lst[6]);      /* y */
DrawTaggedOb(&fm->ob_lst[7]);      /* x */
DrawTaggedOb(&fm->ob_lst[8]);      /* z */
DrawTaggedOb(&fm->ob_lst[9]);      // id
DrawTaggedOb(&fm->ob_lst[11]);     // bias

DrawOb(&fm->ob_lst[10]);           // enabled
}





/*----------------------------------------------------------------------------*/
// display the passed line
/*----------------------------------------------------------------------------*/
void dumpStation(t_station * aStation)
{
int j = 0;
char buf[512];
char latBuf[40];
char lonBuf[40];
char altBuf[40];

if (aStation == (t_station *) NULL)
	{
	j += sprintf(buf+j, "Null Station");
	}
else
	{
	PositionString(lonBuf, latBuf, altBuf, 
                 aStation->pos.lon, 
                 aStation->pos.lat, 
                 aStation->height, 
                 FALSE);

	j += sprintf(buf+j, "name    : %s\n", aStation->name);
	j += sprintf(buf+j, "id      : %d\n", aStation->id);
	j += sprintf(buf+j, "enabled : %d\n", aStation->enabled);
	j += sprintf(buf+j, "display : %d\n", aStation->display);
	j += sprintf(buf+j, "        : %s %s\n", latBuf, lonBuf, altBuf);
	j += sprintf(buf+j, "bias    : %.1lf\n", aStation->bias);
	j += sprintf(buf+j, "delay   : %.1lf\n", aStation->delay);
	}

NotifyUser(-1, -1, __FILE__, buf);
}



/*----------------------------------------------------------------------------*/
// Read the station file
/*----------------------------------------------------------------------------*/
int read_stations(void)
{
FILE *fp;
int k;

if ((fp = fopen(STATION_FILE_NAME, "rb")) == NULL)
  {
  printf("\nCan't open file %s", STATION_FILE_NAME);
  return FALSE;
  }

k = fread( (void *) &navStations, sizeof(navStations), 1, fp);
if (k != 1)
	{
	printf("fread failed at %d in %s\n", __LINE__, __FILE__);
	return BAD_DATA_READ;
	}
StationsAreLoaded = TRUE;

fclose(fp);
return TRUE;
}


/*----------------------------------------------------------------------------*/
// Save the station file
/*----------------------------------------------------------------------------*/
int write_station_file(void)
{
FILE *fp;
int  k;

if ((fp = fopen(STATION_FILE_NAME, "wb")) == NULL)
  {
  printf("\nCan't open file %s", STATION_FILE_NAME);
  fclose(fp);
  return FALSE;
  }

k = fwrite( (char *) &navStations, sizeof(navStations), 1, fp);
if (k == -1)
	{
	printf("Write failed %d\n", k);
	exit(1);
	}

fclose(fp);
return TRUE;
}




/*----------------------------------------------------------------------------*/
// return a pointer to a station
/*----------------------------------------------------------------------------*/
t_station * queryStationByID(int id)
{
int i;

for (i=0; i<MAX_NAV_STATIONS; i++)
	{
	if (navStations[i].id == id)
		return (t_station *) &navStations[i];
	}

return (t_station *) NULL;
}



/*----------------------------------------------------------------------------*/
// return a pointer to a station
/*----------------------------------------------------------------------------*/
t_station * queryStation(int idx)
{
if (idx < 0 || idx > MAX_NAV_STATIONS - 1)
	return (t_station *) NULL;

return (t_station *) &navStations[idx];
}

