/*----------------------------------------------------------------------------*/
// Name: tansdefs.c
// Blame: John Gann
// Start Date: 24 Sep 92
// Manifest constants used for TANS and SuperTANS
/*----------------------------------------------------------------------------*/

#define TANS_VERSION "V1.01"

#define CR  13  /* Carriage return */
#define DISABLE 0x0B
#define FALSE 0
#define FLUSH 0x09
#define FUNC_19 0x13
#define IQCHK 0x0A
#define FF  12  /* Form Feed */
#define LF  10  /* line feed */
#define MAX_COMPORTS 10
#define MAX_PORT_BUF_SIZE    255
#define	MAX_LAT	  1.570796326794897                // PI / 2
#define	MIN_LAT	 -1.570796326794897                // PI / 2
#define	MAX_LON	  3.141592653589793                // PI
#define	MIN_LON	 -3.141592653589793                // PI
#define MPS_TO_KTS 1.9438445
#define KTS_TO_MPS .51444444
#define nyb2byt(a,b) ((((a)&0x000f)<<4)|((b)&0x000f))
#define OQCHK 0x16
#define RD2DG 57.29577951471995 /* radians to degrees multiplier */
#define RSINT 0x14
#define SET_HANDSHAKE 5
#define SUCCESS 0
#define TIME_OUT 0xFFFF
#define TRUE (!FALSE)

#define XMIT_BYTE   0x01
#define READ_BYTE   0x02
#define COMM_STATUS 0x03
#define CONFIGURE   0x04
#define PROTOCOL    0x05
#define ZERO        0x00
#define SEND_BUFFER 0x14
#define READ_STRING 0x15
#define	T_GPS_0 315964800L

#define	PI	 3.141592653589793
#define	TWOPI	 6.283185307179586
#define	HALFPI	 1.570796326794897
#define	T_GPS_0 315964800L

#define METERS_TO_YARDS       1.0936133
#define METERS_TO_FEET        3.28084
#define METERS_TO_SURVEY_FEET ( 3937.0/1200.0 )
#define FEET_TO_METERS        0.3048
#define YARDS_TO_METERS       (1.0 / METERS_TO_YARDS)
#define SURVEY_FEET_TO_METERS ( 1200.0/3937.0 )
#define SECONDS_PER_WEEK      604800L

#define DEG_TO_RAD ( PI/180.0 )
#define RAD_TO_DEG ( 1.0/DEG_TO_RAD ) 

#define	AE_WGS84	6378137.0
#define	E2_WGS84	0.006694380022903416

#define BAD_FILE_OPEN                 -16
#define BAD_DATA_READ                 -17
#define BAD_FILE_POINTER              -19
#define BAD_FILE_SEEK                 -20
#define BAD_PATH_NAME                 -21
#define NOT_ENOUGH_DISK_SPACE         -22
#define BAD_MALLOC                    -23
#define FILE_AT_EOF                   -24
#define NULL_LINE                     -25
#define BAD_TAIL_POINTER              -26
#define DATUM_NOT_FOUND               -27
#define LINES_ALREADY_LOADED          -28
#define BAD_DAY_OF_YEAR               -29
#define BAD_POSITION                  -30
#define BAD_EOL_CONVERSION            -31
#define BAD_UTM_CONVERSION            -32
#define BAD_SOL_CONVERSION            -33
#define BAD_SETUP                     -34
#define NOT_ENOUGH_RANGES             -35
#define BAD_FORM_LOAD                 -36
#define NULL_OBSERVATIONS             -37
#define BAD_FORM_OPEN                 -38
#define NULL_NAV                      -39
#define BAD_FILE_WRITE                -40
#define FILE_NOT_OPEN                 -41
#define FILE_ALREADY_OPEN             -42
#define NULL_HISTORY                  -43
#define NOT_ENOUGH_HISTORY            -44
#define NO_NEW_HISTORY                -45
#define BAD_LATITUDE                  -46
#define BAD_LONGITUDE                 -47
#define NULL_TRACK                    -48
#define UNABLE_TO_CREATE_TMPFILE      -49
#define NOT_TIME_YET                  -50
#define LOGGING_NOT_ENABLED           -51
#define BAD_OFFSET                    -52
#define OLD_NAV_DATA                  -53
#define BAD_TRACKPOINT                -54
#define BAD_DDMU540                   -55
#define NO_LIVE_WINDOWS_BOARD         -56
#define RUBBER_SHAPE_LIMIT_ERR        -57
#define NO_MATCHING_FILES             -58
#define NO_MEMORY                     -59
#define NO_FISH_NAV                   -60
#define SITES_ALREADY_LOADED          -61
#define NULL_SITE                     -62
#define BAD_PLOTTER_READ              -63
#define NULL_COM                      -64
#define SERIAL_IO_TIMEOUT             -65
#define TANS_NOT_INITIALIZED          -66

// control lines constants
#define XON_XOFF 1
#define XON     17
#define XOFF    18
#define ETX                      0x03
#define DLE                      0x10
#define COM1                     0                     // Async Adapter COM1:
#define COM2                     1                     // Async Adapter COM2:

#define MAX_PACKET_SIZE 1024
#define COMPUTE_NEW_ZONE 1
#define USE_PASSED_ZONE 0

// More tans error codes
#define BAD_FIX_DATA                    -201
#define OLD_FIX_DATA                    -202
#define BAD_GPS_HEALTH                  -203
#define HIGH_PDOP                          3
#define COM_PORT_OUT_OF_RANGE           -204
#define UNABLE_TO_INIT_EC_COMM          -205
#define BAD_SAT_DOP_MODE                -206
#define BAD_TIME_PKT                    -207

#define PI_ICD200 3.1415926535898
#define SPEED_LIGHT 2.99792458e8
#define SQTU_WGS84 1.9964981843217e7
#define RECEIVER_TIMEOUT 15L
#define SERIAL_PORT_THRESHOLD 2000

// Packet ID's 
#define		GPS_SYS_TIME_PKT	0x41
#define		VEL_FIX_ENU_PKT		0x56
#define		MACHINE_CODE_PKT	0x4B
#define		ALM_HEALTH_PKT		0x49
#define		ALM_PKT						0x40
#define		GPS_SYS_TIME_PKT	0x41
#define		POS_FIX_ECEF_PKT	0x42
#define		VEL_FIX_ECEF_PKT	0x43
#define		SAT_DOP_MODE_PKT	0x44
#define		POS_LLA_DBL_PKT		0x84
#define		POS_FIX_LLA_PKT		0x4A
#define		HEALTH_PKT				0x46
#define		POS_ECEF_DBL_PKT	0x83
#define		SOFT_VER_PKT			0x45
#define		EPH_STATUS_PKT		0x5B
#define		IO_OPTIONS_PKT		0x55
#define		RAW_MSMT_PKT      0x5A
#define		SAT_TRK_PKT       0x5C
#define		TEST_PARAM_PKT    0x5F
#define		OP_PRAMS_PKT		  0x4C
#define		SNR_PKT      		  0x47
#define		A_D_PKT			      0x53
#define		POS_FIX_ECEF_PKT	0x42
#define		LAST_FIX_INFO_PKT	0x57

// Navigation constants
#define MAX_WAYPTS        200
#define MAX_ROUTES         50
#define MAX_LEGS           15
#define LLA_POS                 1
#define UTM_POS                 2
#define MGRS_POS                3
#define ENU_VEL                 1
#define CRS_GS_VEL              2

// Packet Requests
#define SET_IO_OPTIONS           0x35
#define SET_OPERATING_PARAMS     0x2C
#define REQ_FIRMWARE             0x1F

#define PACKET_IN_PROGRESS 2
#define NEW_INFO           1


// Macros
#define HURL(msg, value)                                                      \
                    {                                                         \
                    printf("%s:%d:%s %d\n", __FILE__, __LINE__, msg, value);  \
                    termTans(msg);                                            \
                    }

