/*----------------------------------------------------------------------------*/
// Name: tanstype.c
// Blame: John Gann
// Start Date: 24 Sep 92
// Typedefs used for TANS and SuperTANS
/*----------------------------------------------------------------------------*/

typedef struct tans_com_struct
	{
	int port;
	unsigned int baud;
	unsigned char parity;
	unsigned char data;
	unsigned char stop;
	time_t timeOfLastPkt;
	} t_tansCom;

typedef struct	
  {
	unsigned char id; 
	float a;                           // SMA meters
	float e;                           // eccentricity
	float dx;                          // meters
	float dy;                          // meters
	float dz;                          // meters
	} t_tansDatum;

typedef struct	
  {
	int   dynamics; 
	float elvMask;
	float dopMask;
	float snrMask;
	float dopSwitch;
	} t_opPrams;

typedef struct	
  {
	unsigned char majorVersion;
	unsigned char minorVersion;
	unsigned char month;
	unsigned char day;
	unsigned char year;
	} t_version;

typedef struct	
  {
	t_version navProc;
	t_version sigProc;
	} t_firmware;

typedef struct enu_struct
  {
	float east;                    // velocity east in m/s 
	float north;                   // velocity north in m/s 
	float up;                      // velocity up in m/s 
	float biasRate;                // meters / Sec
	float timeOfFix;               // sec
	} t_enuVel;

typedef struct crs_struct
  {
	float crs;                     // degrees 
	float spd;                     // meters / sec
	float climbRate;               // meters / Sec
	float biasRate;                // meters / Sec
	float timeOfFix;               // sec
	} t_crsGsVel;

typedef struct lla_struct
  {
	float lat;                     // radians of latitude
	float lon;                     // radians of longitude
	float alt;                     // meters of altitude
	} t_llaPos;

typedef struct tans_utm_struct
  {
	char  zone[2];                 // Grid zone
	float easting;                 // meters
	float northing;                // meters
	float alt;                     // meters
	} t_utmPos;

typedef struct tans_mgr_struct
  {
	char  zone[2];                 // Grid zone
	char  squareId[2];             // Grid zone
	float easting;                 // meters
	float northing;                // meters
	float alt;                     // meters
	} t_mgrPos;

typedef union pos_union
  {
	t_llaPos lla;                 // lat lon alt
	t_utmPos utm;                 // UTM
	t_mgrPos mgr;                 // MGRS
	} t_multiCoordPos;
	
typedef union vel_union
  {
	int coordType;                // 1=ENU, 2=CRS_GS
	t_enuVel   enu;               // East, north up
	t_crsGsVel crs;               // Course and Ground Speed
	} t_multiCoordVel;

typedef struct waypoint_struct
	{
	int coordType;                 // 1=LLA, 2=UTM, 3=MGRS
	int id;                        // waypoint number
	char label[13];                // name of waypoint
	t_multiCoordPos pos;           // position in one of 3 coord systems
	int timeFlag;                  // ??
	} t_wayPoint;

typedef struct route_struct
	{
	int id;
	int switchMode;                  // leg switching mode
	float swRadius;                  // leg switching radius for mode 2
	int wpts[MAX_LEGS+1];            // maximum # of Wpts in a route is 16
	} t_route;

typedef struct tans_nav_struct
	{
	int mode;                                // direct ??
	float spd;                               // speed in knots
	float crs;                               // course made good in radians
	float xte;                               // Cross track error meters
	float vte;                               // Vertical Error
	float her;                               // Heading error
	float ete;                               // Est. time en route
	float dtg;                               // Dist to final waypoint
	float brg;                               // Bearing to next waypoint
	float rng;                               // Range to next waypoint
	float ttg;                               // Dist to next waypoint
	int toWaypoint;                          // current to waypoint
	char curRoute;                           // current Route
	char curLeg;                             // current Leg in route
	int nextWaypoint;                        // Next Waypoint
	int finalWaypoint;                       // Last Waypoint in route
	t_wayPoint wpt[MAX_WAYPTS];              // list of waypoints
	t_route    rte[MAX_ROUTES];              // list of routes
	} t_tansNav;

typedef struct tans_cfg_struct
	{
	int initialized;                         // has library been initialized
	int lastErr;                             // Last reported error
	t_tansCom com;                           // communications
	t_opPrams opPrams;                       // operating parameters
	t_firmware firmware;                     // Nav & Sig Proc firmware revision
	t_tansDatum datum;                       // current Datum
	} t_tansCfg;      

typedef struct tansGPS_struct
	{
	double gpsTow;                           // time of week
	int    gpsWeek;                          // week number
	t_multiCoordPos pos;                     // pos in one of 3 coord systems
	t_multiCoordVel vel;                     // velocity in one of 2 coord systems
	double timeOfFix;                        // time of last fix
	unsigned char health;                    // receiver health
	unsigned char status;                    // Tans status
	unsigned char machineCodeId;             // Rcvr Id
	double pdop;                             // PDOP
	double hdop;                             // HDOP
	double vdop;                             // VDOP
	int fixMode;                             // 2D, 3D etc
	int numSatsUsed;                         // # of Sats used in fix
	int prnsUsed[8];                         // SV ids used in fix
	int coordSys;                            // 1=LLA, 2=UTM, 3=MGRS
	} t_tansGps;      

typedef struct tans_struct
	{
	t_tansCfg cfg;                           // Tans config. params
	t_tansGps gps;                           // GPS parameters
	t_tansNav nav;                           // navigation struct
	} t_tans;

typedef union val_union
 {
 char   ch[8];
 int    i;
 float  f;
 double d;
 } t_uval;


