/*----------------------------------------------------------------------------*/
// File: tracking.c USGS Navigation, allow user to view various sensor tracks
// Blame: John Gann
// Start Date: 4 May 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "mnutils.h"
#include "nav_utils.h"
#include "sensor.h"
#include "lines.h"
#include "tracking.h"

// private
static char *names [ MAX_COMPORTS ];
static int sensorCount = 0;

// externals
extern char RefreshRequired;


/*---------------------------------------------------------------------------*/
// Select the Active Sensor
/*---------------------------------------------------------------------------*/
int doTrackingFm(ob_desc * ob)
{
return pickDisplayedSensors(ob, "Select Sensors for Main Display");
}


/*------------------------------------------------------------------*/
// Pick sensors to be tracked on display
/*------------------------------------------------------------------*/
int pickDisplayedSensors(ob_desc * ob, char * title)
{
fm_desc *lf;
int max_length = 0, i;
int cur_slot, top_slot, retv;
char marks[MAX_LINES];
char * err_msg =   "Unable to match nav sensor\n";

memset((void *) marks, ' ', sizeof(marks));

/* load the sensor names */
if ((sensorCount = loadSensorNames(names, &max_length)) <= 0)
   {
   NotifyUser(-1, -1, "Yo", "No Sensors are Enabled." );
   return -1;
   }

// get the current display flags
sensorDisplayFlags(marks, GET);

/* create a list form */
lf = CreateChkListForm( title, 7, 4, max_length+2, 7);
lf->ob_lst[9].label = "Okay";
lf->ob_lst[9].flags |= IS_EXITABLE;
lf->ob_lst[9].key = 'O';
lf->ob_lst[9].state = 8;

lf->ob_lst[10].label = "All";
lf->ob_lst[10].flags |= IS_EXITABLE;
lf->ob_lst[10].key = 'A';
lf->ob_lst[10].state = 8;

lf->ob_lst[11].label = "None";
lf->ob_lst[11].flags |= IS_EXITABLE;
lf->ob_lst[11].key = 'N';
lf->ob_lst[11].state = 8;

lf->ob_lst[12].label = "Cancel";
lf->ob_lst[12].flags |= IS_EXITABLE;
lf->ob_lst[12].key = 'C';
lf->ob_lst[12].state = 8;

cur_slot = top_slot = 0;

/* open a form to put the list on */
OpenForm( lf, 10, ob->area.Ymax);

do 
	{
	retv = ProcessChkListForm( lf, names, marks, sensorCount, &top_slot);
	switch ( retv ) 
		{
		case 9 :        // Okay
			RefreshRequired = TRUE;
			sensorDisplayFlags(marks, SET);
			break;

		case 10:        // All On
			for (i=0; i< sensorCount; i++)
				marks[i] = '+';
			top_slot = 0;
			DrawForm(lf);
			break;
	
		case 11 :       // All Off
			for (i=0; i< sensorCount; i++)
				marks[i] = ' ';
			top_slot = 0;
			DrawForm(lf);
			break;
  	 
		case 12 :       // Cancel
			break; 
		}
	} while ( (retv != 9) && (retv != NOT_SET) && ( retv != 12 ) );

// clean up alloced memory
i = sensorCount;
while ( i ) 
	free ( names [ --i ] );

CloseForm   ( lf );
DestroyForm ( lf );

if (retv == 9)
	{
	return cur_slot;
	}
else
	{
	return -1;
	}
}


/*----------------------------------------------------------------------------*/
// Load the display flag from the active sensor list
/*----------------------------------------------------------------------------*/
int sensorDisplayFlags(char marks[], int action)
{
int i;
int port;
t_setup * config = QuerySetup();
int count = 0;

for(i=0; i<sensorCount; i++)
	{
	port = atoi(&names[i][3]) - 1;
	if (port >=0 && port < MAX_COMPORTS)
		{
		if (action == GET)
			{
			marks [ i ] = (char) ((config->displaySensors[port]) ? '+' : ' ');
			}
		else
			{
			// Set the display flag
			if (marks[i] == '+')
				{
				config->displaySensors[port] = TRUE;
				}
			else
				{
				config->displaySensors[port] = FALSE;
				}
			}      // end of if action ==
		}     // end of if port >=0 ...
	
	if (marks[i] == '+')
		count++;
	}    // end of for loop

return count;
}
